/*
 * CameraInfo.cpp
 *
 *  Created on: Sep 13, 2010
 *      Author: ltwardon
 */

#define _USE_MATH_DEFINES
#include <math.h>

#include "CameraInfo.h"

using namespace airobots;

const map<CameraInfo::CameraModel, CameraInfo> CameraInfo::info = CameraInfo::createInfo();

map<CameraInfo::CameraModel, CameraInfo> CameraInfo::createInfo() {

	map<CameraInfo::CameraModel,CameraInfo> info;
	info[CameraInfo::SONY_EVI_D31] = CameraInfo(
			12, // max zoom factor
			4.3, 48.8, // angle of view
			752, 585, // resolution
			-100.0, 100.0, // angle pan
			-25.0, 25.0); // angle tilt
	info[CameraInfo::LOGITECH_PRO_9000] = CameraInfo(
			1, // max zoom factor
			74.0, 74.0, // angle of view
			800, 600, // resolution
			0.0, 0.0, // angle pan
			0.0, 0.0); // angle tilt
	info[CameraInfo::POINTGRAY_GRAS_20S4C_C_0_3M] = CameraInfo(
			1, // max zoom factor
			74.0, 74.0, // angle of view
			640, 480, // resolution
			0.0, 0.0, // angle pan
			0.0, 0.0); // angle tilt
	info[CameraInfo::POINTGRAY_GRAS_20S4C_C_1_4M] = CameraInfo(
			1, // max zoom factor
			74.0, 74.0, // angle of view
			1384, 1036, // resolution
			0.0, 0.0, // angle pan
			0.0, 0.0); // angle tilt
	info[CameraInfo::POINTGRAY_GRAS_20S4C_C_2_0M] = CameraInfo(
			1, // max zoom factor
			74.0, 74.0, // angle of view
			640, 480, // resolution
			1624.0, 1224.0, // angle pan
			0.0, 0.0); // angle tilt
	return info;

}

CameraInfo CameraInfo::getInfo(CameraInfo::CameraModel model){
	return info.at(model);
}

double CameraInfo::calculateAngleOfView(CameraModel camera, double zoom) {

	if (info.at(camera).getMaxAngleOfViewHRad() == info.at(camera)
			.getMinAngleOfViewHRad()) {
		return info.at(camera).getMaxAngleOfViewHRad();
	}

	double angle = (1 - zoom)
			* (info.at(camera).getMaxAngleOfViewHRad() - info.at(camera)
					.getMinAngleOfViewHRad())
			+ info.at(camera).getMinAngleOfViewHRad();
	return angle;
}

double CameraInfo::degreeToRadiant(double angle) {
	return angle * M_PI / 180.0;
}

double CameraInfo::radiantToDegree(double angle) {
	return angle * 180.0 / M_PI;
}

CameraInfo::CameraInfo(){

}

CameraInfo::CameraInfo(int maxZoomFactor, double minAngleOfViewH,
			double maxAngleOfViewH, int resolutionX, int resolutionY,
			double minAnglePan, double maxAnglePan, double minAngleTilt,
			double maxAngleTilt) {
	this->maxZoomFactor = maxZoomFactor;
	this->minAngleOfViewH = minAngleOfViewH;
	this->maxAngleOfViewH = maxAngleOfViewH;
	this->resolutionX = resolutionX;
	this->resolutionY = resolutionY;
	this->minAnglePan = minAnglePan;
	this->maxAnglePan = maxAnglePan;
	this->minAngleTilt = minAngleTilt;
	this->maxAngleTilt = maxAngleTilt;
}

double CameraInfo::getMaxAngleOfViewHDeg() const
{
    return maxAngleOfViewH;
}

double CameraInfo::getMaxAngleOfViewHRad() const
{
    return degreeToRadiant(maxAngleOfViewH);
}

double CameraInfo::getMaxAnglePanDeg() const
{
    return maxAnglePan;
}

double CameraInfo::getMaxAnglePanRad() const
{
    return degreeToRadiant(maxAnglePan);
}

double CameraInfo::getMaxAngleTiltDeg() const
{
    return maxAngleTilt;
}

double CameraInfo::getMaxAngleTiltRad() const
{
    return degreeToRadiant(maxAngleTilt);
}

int CameraInfo::getMaxZoomFactor() const
{
    return maxZoomFactor;
}

double CameraInfo::getMinAngleOfViewHDeg() const
{
    return minAngleOfViewH;
}

double CameraInfo::getMinAngleOfViewHRad() const
{
    return degreeToRadiant(minAngleOfViewH);
}

double CameraInfo::getMinAnglePanDeg() const
{
    return minAnglePan;
}

double CameraInfo::getMinAnglePanRad() const
{
    return degreeToRadiant(minAnglePan);
}

double CameraInfo::getMinAngleTiltDeg() const
{
    return minAngleTilt;
}

double CameraInfo::getMinAngleTiltRad() const
{
    return degreeToRadiant(minAngleTilt);
}

int CameraInfo::getResolutionX() const
{
    return resolutionX;
}

int CameraInfo::getResolutionY() const
{
    return resolutionY;
}
