/*
 * CameraInfo.h
 *
 *  Created on: Sep 13, 2010
 *      Author: ltwardon
 */

#ifndef CAMERAINFO_H_
#define CAMERAINFO_H_

#include <map>

using namespace std;

namespace airobots{

class CameraInfo {

public:

	enum CameraModel{
		SONY_EVI_D31,
		LOGITECH_PRO_9000,
		POINTGRAY_GRAS_20S4C_C_0_3M,
		POINTGRAY_GRAS_20S4C_C_1_4M,
		POINTGRAY_GRAS_20S4C_C_2_0M
	};

	static CameraInfo getInfo(CameraModel model);
	static double calculateAngleOfView(CameraModel camera, double zoom);
	static double degreeToRadiant(double angle);
	static double radiantToDegree(double angle);

	CameraInfo();
	CameraInfo(int maxZoomFactor, double minAngleOfViewH,
			double maxAngleOfViewH, int resolutionX, int resolutionY,
			double minAnglePan, double maxAnglePan, double minAngleTilt,
			double maxAngleTilt);
	double getMaxAngleOfViewHDeg() const;
	double getMaxAngleOfViewHRad() const;
    double getMaxAnglePanDeg() const;
    double getMaxAnglePanRad() const;
    double getMaxAngleTiltDeg() const;
    double getMaxAngleTiltRad() const;
    int getMaxZoomFactor() const;
    double getMinAngleOfViewHDeg() const;
    double getMinAngleOfViewHRad() const;
    double getMinAnglePanDeg() const;
    double getMinAnglePanRad() const;
    double getMinAngleTiltDeg() const;
    double getMinAngleTiltRad() const;
    int getResolutionX() const;
    int getResolutionY() const;

private:

    static const map<CameraModel, CameraInfo> info;
	static map<CameraModel, CameraInfo> createInfo();

	int maxZoomFactor;
	double minAngleOfViewH;
	double maxAngleOfViewH;
	double minAnglePan;
	double maxAnglePan;
	double minAngleTilt;
	double maxAngleTilt;
	int resolutionX;
	int resolutionY;

};

}

#endif /* CAMERAINFO_H_ */
