/*
 * CameraTargetData.h
 *
 *  Created on: Aug 6, 2009
 *      Author: jwienke
 */

#ifndef CAMERATARGETDATA_H_
#define CAMERATARGETDATA_H_

#include "Type.h"

namespace airobots {

class CameraTargetData: public Type {

public:

	static std::string getBaseTag();

	static std::string getTemplate();

	CameraTargetData();

	/**
	 * Returns the distance of the target to focus in laser coordinates (mm).
	 */
	double getDistance() const;
	void setDistance(double distance);

	/**
	 * Returns the height of the target to focus in mm over floor.
	 */
	double getHeight() const;
	void setHeight(double height);

	/**
	 * Returns the azimuth angle of the target to focus relative to the camera in
	 * rad.
	 * TODO what is angle 0?
	 */
	double getAzimuth() const;
	void setAzimuth(double azimuth);

	/**
	 * Returns the desired zoom level as percent of the maximum zoom of the
	 * camera [0,1].
	 */
	double getZoom() const;
	void setZoom(double zoom);

	void sanitizeLocation(xmltio::Location &loc) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, CameraTargetData &type);

private:

	double distance;
	double height;
	double azimuth;

	double zoom;

};

}

#endif /* CAMERATARGETDATA_H_ */
