#include "GlobalPlan.h"

using namespace xmltio;
using namespace airobots;
using namespace std;

std::string GlobalPlan::getBaseTag() {
	return "GLOBALPLAN";
}

std::string GlobalPlan::getTemplate() {
	return "<" + getBaseTag() + "><GENERATOR/>" + Timestamp::getTemplate()
			+ "</" + getBaseTag() + ">";
}

GlobalPlan::GlobalPlan() : Type("unknown") {
}

GlobalPlan::GlobalPlan(std::string generator) :
	Type(generator) {

}

void GlobalPlan::addWaypoint(const NavigationGoalData waypoint) {
	waypoints.push_back(waypoint);
}

void GlobalPlan::removeLastWayoint() {
	waypoints.erase(--(waypoints.end()));
}

std::list<NavigationGoalData>::const_iterator GlobalPlan::begin() const {
	return waypoints.begin();
}

std::list<NavigationGoalData>::const_iterator GlobalPlan::end() const {
	return waypoints.end();
}

void GlobalPlan::clear() {
	waypoints.clear();
}

size_t GlobalPlan::size() const {
	return waypoints.size();
}

bool GlobalPlan::empty() const {
	return waypoints.empty();
}

void GlobalPlan::sanitizeLocation(xmltio::Location &loc) const {

	XPath xp(NavigationGoalData::getBaseTag());
	while (XPathIterator i = xp.evaluate(loc)) {
		Location l(*i);
		loc.remove(l);
	}

}

void GlobalPlan::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	for (list<NavigationGoalData>::const_iterator it = begin(); it
			!= end(); it++) {

		Location newWaypoint(NavigationGoalData::getTemplate(),
				NavigationGoalData::getBaseTag());
		newWaypoint = *it;

		loc.add(newWaypoint);

	}

}

void GlobalPlan::fromLocation(const xmltio::Location &loc_, GlobalPlan &plan) {

	Type::fromLocation(loc_, plan);

	for (XPathIterator wpIter =
			XPath(NavigationGoalData::getBaseTag()).evaluate(loc_); wpIter; ++wpIter) {

		NavigationGoalData wp = extract<NavigationGoalData> (*wpIter);
		plan.addWaypoint(wp);

	}

}
