/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; c-file-offsets: ((substatement-open . 0)) -*- */
/**
 * @file     laserdata.cc
 * @author   Erik Weitnauer, Thorsten Spexard
 * @date     15.08.2005
 * @brief    defining and converting laser data in C++ stucts and XML
 */

#include <stdio.h>
#include <stdlib.h>

#include <cmath>
#include <sstream>

#include "LaserData.h"

using namespace std;
using namespace xmltio;
using namespace airobots;

std::string LaserData::getBaseTag() {
	return "LASERDATA";
}

std::string LaserData::getTemplate() {
	return std::string("<") + getBaseTag() + ">" + "<GENERATOR/>"
			+ Timestamp::getTemplate() + MicroTimestamp::getTemplate()
			+ "<MEASUREMENTS num=\"\"/>" + "</" + getBaseTag() + ">";
}

LaserData::LaserData() :
	Type("unknown") {

}

unsigned int LaserData::getNumLaserPoints() const {
	return points.size();
}

const std::vector<float> & LaserData::getLaserPoints() const {
	return points;
}

void LaserData::setLaserPoints(std::vector<float> laserPoints) {
	this->points = laserPoints;
}

void LaserData::sanitizeLocation(xmltio::Location &loc) const {
//	std::cout << "LaserData sanitizing location" << std::endl;
}

void LaserData::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	loc[XPath(MicroTimestamp::getBaseTag())] = getReadTime();

	loc[XPath("MEASUREMENTS/@num")] = getNumLaserPoints();

	// write all int vals in a string, limiter is space
	stringstream pointStream;
	vector<float> points = getLaserPoints();
	for (vector<float>::const_iterator it = points.begin(); it != points.end(); ++it) {
		pointStream << (int) round(*it * 1000) << " ";
	}
	loc[XPath("MEASUREMENTS")] = pointStream.str();

}

void LaserData::fromLocation(const xmltio::Location &loc_, LaserData &ld) {

	string strData;
	Type::fromLocation(loc_, ld);

	// read time
	ld.setReadTime(extract<MicroTimestamp> (loc_[XPath(
			MicroTimestamp::getBaseTag())]));

	// process measurements
	unsigned int numPoints = extract<unsigned int> (loc_[XPath(
			"MEASUREMENTS/@num")]);

	string pointsString = extract<string> (loc_[XPath("MEASUREMENTS")]);
	stringstream pointsStream;
	pointsStream << pointsString;
	vector<float> points;
	float currentPoint;
	while (pointsStream >> currentPoint) {
		points.push_back(currentPoint / 1000.0);
	}
	if (points.size() != numPoints) {
		stringstream msg;
		msg << "Parsed " << points.size() << " points but header required "
				<< numPoints << " points.";
		throw ParsingException(msg.str());
	}
	ld.setLaserPoints(points);

}
