/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; c-file-offsets: ((substatement-open . 0)) -*- */
/**
 * @file     laserdata.h
 * @author   Erik Weitnauer, Thorsten Spexard
 * @date     15.08.2005
 * @brief    defining and converting laser data in C++ stucts and XML
 */

#ifndef LASERDATA_CPP
#define LASERDATA_CPP

#include <string>
#include <vector>

#include <sys/time.h>

#include <xmltio/xmltio.hpp>

#include "Type.h"
#include "Ticked.h"

namespace airobots {

/**
 * C++ struct for laser data
 */
class LaserData: public Type, public Ticked {

public:

	static std::string getBaseTag();

	static std::string getTemplate();

	LaserData();

	unsigned int getNumLaserPoints() const;

	const std::vector<float> & getLaserPoints() const;
	void setLaserPoints(std::vector<float> laserPoints);

	void sanitizeLocation(xmltio::Location &loc) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, LaserData &type);

private:

	/**
	 * Time at which these data were read.
	 */
	timeval readTime;

	/**
	 * array with distance measurements generated by laser range finder,
	 * distances in m relative to the robot
	 */
	std::vector<float> points;

};

}

#endif
