/*
 * LaserInfo.h
 *
 *  Created on: Sep 17, 2010
 *      Author: ltwardon
 */

#ifndef LASERINFO_H_
#define LASERINFO_H_

#include <map>
#include <exception>
#include <string>

using namespace std;

namespace airobots{

class LaserInfo {

public:

	enum ScannerModel {
		SICK_LMS200,
		HOKUYO_UBG_04LX_F01
	};

	class OutOfRangeException : public exception {
	private:
		std::string message;
	public:
		OutOfRangeException(string message = "requested angle is out of laser range!") {
			this->message = message;
		}

		~OutOfRangeException() throw() {}

		virtual const char* what() const throw() {
			return message.c_str();
		}
	};

	static LaserInfo getInfo(ScannerModel model);
	static double degreeToRadiant(double angle);
	static double radiantToDegree(double angle);
	static int calcScanIndex(ScannerModel scannerModel, double angle) throw(OutOfRangeException);

	LaserInfo();
	LaserInfo(double range, int numScans, double maxDistance);
	double getRangeDeg();
	double getRangeRad();
	double getMaxDistance();
	int getNumScans();

private:

    static const map<ScannerModel, LaserInfo> info;
	static map<ScannerModel, LaserInfo> createInfo();

	double range;
	double maxDistance;
	int numScans;

};

}

#endif /* LASERINFO_H_ */
