/*
 * MicroTimestamp.h
 *
 *  Created on: Aug 6, 2009
 *      Author: jwienke
 */

#ifndef MICROTIMESTAMP_H_
#define MICROTIMESTAMP_H_

#include <string>
#include <sstream>

#include <sys/time.h>

#include <xmltio/xmltio.hpp>

namespace airobots {

/**
 * A timestamp with microseconds precision.
 */
class MicroTimestamp {
public:

	static timeval timevalFromDouble(double time);

	static std::string getBaseTag();

	static std::string getTemplate();

	MicroTimestamp();
	MicroTimestamp(timeval readtime);
	MicroTimestamp(double time);
	MicroTimestamp(time_t seconds, suseconds_t microSeconds);

	timeval getTimeval() const;
	time_t getSeconds() const;
	suseconds_t getMicroSeconds() const;

	void set(timeval time);

	bool operator==(const MicroTimestamp & other) const;

private:
	timeval readtime;

};

std::ostream& operator<<(std::ostream& o, MicroTimestamp const& timestamp);

}

namespace xmltio {

void convert(const airobots::MicroTimestamp& ld, Location& loc);

template<>
airobots::MicroTimestamp xmltio::extract<airobots::MicroTimestamp>::get() const;

}

#endif /* MICROTIMESTAMP_H_ */
