#include "ObjectData.h"

using namespace xmltio;
using namespace airobots;
using namespace std;

std::string ObjectData::getBaseTag() {
	return "OBJECT";
}

std::string ObjectData::Hypothesis::getBaseTag() {
	return "HYPOTHESIS";
}

std::string ObjectData::getTemplate() {
	return "<" + getBaseTag() + "><CLASS/></" + getBaseTag() + ">";
}

std::string ObjectData::Hypothesis::getTemplate() {
	return "<" + getBaseTag() + "><GENERATOR/>" + Timestamp::getTemplate()
			+ "<RATING><RELIABILITY value=\"\"/></RATING></" + getBaseTag()
			+ ">";
}

ObjectData::ObjectData() :
	Type("unknown") {
}

ObjectData::Hypothesis::Hypothesis() :
	Type("unknown") {
}

ObjectData::ObjectData(std::string generator) :
	Type(generator) {
}

ObjectData::Hypothesis::Hypothesis(std::string generator) :
	Type(generator) {
}

ObjectData::~ObjectData() {
}

ObjectData::Hypothesis::Hypothesis(Hypothesis &h) :
	Type(), reliability(h.reliability) {

	setTimestamp(h.getTimestamp());
	setGenerator(h.getGenerator());
}

void ObjectData::sanitizeLocation(xmltio::Location &loc) const {
	XPath xp(ObjectData::Hypothesis::getBaseTag());
	while (XPathIterator i = xp.evaluate(loc)) {
		Location l(*i);
		loc.remove(l);
	}
}

void ObjectData::Hypothesis::sanitizeLocation(xmltio::Location&) const {

}

void ObjectData::fillInto(xmltio::Location &loc) const {
	for (vector<Hypothesis*>::const_iterator it = hypotheses.begin(); it
			!= hypotheses.end(); it++) {

		Hypothesis h(**it);
		Location
				hyp(Hypothesis::getTemplate(), XPath(Hypothesis::getBaseTag()));
		h.fillInto(hyp);
		loc.add(hyp);
	}
	loc[XPath("CLASS")] = getClassLabel();
}

void ObjectData::Hypothesis::fillInto(xmltio::Location &loc) const {
	Type::fillInto(loc);
	loc[XPath("RATING/RELIABILITY/@value")] = getReliability();
}

void ObjectData::fromLocation(const xmltio::Location &loc_, ObjectData &object) {

	for (XPathIterator iter = XPath(Hypothesis::getBaseTag()).evaluate(loc_); iter; ++iter) {
		Hypothesis* hyp = new Hypothesis();
		Hypothesis::fromLocation(*iter, *hyp);
		object.addHypothesis(hyp);
	}

	string s = extract<string> (loc_[XPath("CLASS")]);
	object.setClassLabel(s);
}

void ObjectData::Hypothesis::fromLocation(const xmltio::Location &loc_,
		Hypothesis &object) {

	Type::fromLocation(loc_, object);
	double r = extract<double> (loc_[XPath("RATING/RELIABILITY/@value")]);
	object.setReliability(r);
}

string ObjectData::getClassLabel() const {
	return classLabel;
}
void ObjectData::setClassLabel(const string &label) {
	classLabel = label;
}

double ObjectData::Hypothesis::getReliability() const {
	return reliability;
}
void ObjectData::Hypothesis::setReliability(const double &reliability) {
	Hypothesis::reliability = reliability;
}

bool ObjectData::operator==(const ObjectData &other) const {

	bool isHyp = true;

	vector<Hypothesis*> h0 = getHypotheses();
	vector<Hypothesis*> h1 = other.getHypotheses();

	vector<Hypothesis*>::iterator it0 = h0.begin();
	for (vector<Hypothesis*>::iterator it1 = h1.begin(); it1 != h1.end(); it1++) {

		Hypothesis* hyp0 = *it0;
		Hypothesis* hyp1 = *it1;
		if (!(*hyp0 == *hyp1)) {
			isHyp = false;
		}
		it0++;
	}

	return (other.getClassLabel() == classLabel) && isHyp;
}

bool ObjectData::Hypothesis::operator==(const Hypothesis &other) const {
	return (other.getGenerator() == getGenerator()) && (other.getTimestamp()
			== getTimestamp()) && (other.getReliability() == getReliability());
}

void ObjectData::addHypothesis(Hypothesis* h) {
	hypotheses.push_back(h);
}

void ObjectData::clearHypotheses() {
	hypotheses.clear();
}

vector<ObjectData::Hypothesis*> ObjectData::getHypotheses() const {
	return hypotheses;
}
