#include "ObjectPositionData.h"

using namespace xmltio;
using namespace airobots;
using namespace std;

std::string ObjectPositionData::getBaseTag() {
	return "OBJECTPOSITION";
}

std::string ObjectPositionData::Hypothesis::getBaseTag() {
	return "HYPOTHESIS";
}

std::string ObjectPositionData::getTemplate() {
	return "<" + getBaseTag() + "><CLASS/></" + getBaseTag() + ">";
}

std::string ObjectPositionData::Hypothesis::getTemplate() {
	return "<" + getBaseTag() + "><GENERATOR/>" + Timestamp::getTemplate()
			+ "<RATING><RELIABILITY value=\"\"/></RATING><SENSORTYPE/>"
			+ "<PRECISEPOLYGON kind=\"\" ref=\"\"/></" + getBaseTag() + ">";
}

ObjectPositionData::ObjectPositionData() :
	ObjectData() {
}

ObjectPositionData::Hypothesis::Hypothesis() :
	ObjectData::Hypothesis() {
}

ObjectPositionData::ObjectPositionData(std::string generator) :
	ObjectData(generator) {
}

ObjectPositionData::Hypothesis::Hypothesis(std::string generator) :
	ObjectData::Hypothesis(generator) {
}

void ObjectPositionData::sanitizeLocation(xmltio::Location &loc) const {
	XPath xp(Hypothesis::getBaseTag());
	while (XPathIterator i = xp.evaluate(loc)) {
		Location l(*i);
		loc.remove(l);
	}
}

void ObjectPositionData::Hypothesis::sanitizeLocation(xmltio::Location &loc) const {
	XPath xp("POINT2D");
	while (XPathIterator i = xp.evaluate(loc)) {
		Location l(*i);
		loc.remove(l);
	}
}

void ObjectPositionData::fillInto(xmltio::Location &loc) const {

	ObjectData::fillInto(loc);
	ObjectData::sanitizeLocation(loc);

	vector<ObjectData::Hypothesis*> h = getHypotheses();
	for (vector<ObjectData::Hypothesis*>::const_iterator it = h.begin(); it
			!= h.end(); it++) {

		try {
			Hypothesis* h0 = dynamic_cast<Hypothesis*> (*it);

			Location hyp(Hypothesis::getTemplate(), XPath(
					Hypothesis::getBaseTag()));

			h0->fillInto(hyp);
			loc.add(hyp);
		} catch (bad_cast e) {
			continue;
		}
	}
}

void ObjectPositionData::Hypothesis::fillInto(xmltio::Location &loc) const {

	ObjectData::Hypothesis::fillInto(loc);

	Location polyLoc = Location(loc, "PRECISEPOLYGON");
	polyLoc[XPath("@kind")] = coordinateKind;
	polyLoc[XPath("@ref")] = reference;

	for (vector<pair<double, double> >::const_iterator it = polygon.begin(); it
			!= polygon.end(); it++) {
		Location newPoint("<POINT2D x=\"\" y=\"\" />", "POINT2D");
		newPoint[XPath("@x")] = it->first;
		newPoint[XPath("@y")] = it->second;
		polyLoc.add(newPoint);
	}
}

void ObjectPositionData::fromLocation(const xmltio::Location &loc_,
		ObjectPositionData &object) {

	ObjectData::fromLocation(loc_, object);

	for (XPathIterator iter = XPath(Hypothesis::getBaseTag()).evaluate(loc_); iter; ++iter) {
		Hypothesis* hyp = new Hypothesis();
		Hypothesis::fromLocation(*iter, *hyp);
		object.clearHypotheses();
		object.addHypothesis(hyp);
	}
}

void ObjectPositionData::Hypothesis::fromLocation(const xmltio::Location &loc_,
		Hypothesis &object) {

	ObjectData::Hypothesis::fromLocation(loc_, object);

	object.setCoordinateKind(extract<string> (
			loc_[XPath("PRECISEPOLYGON/@kind")]));
	object.setReference(extract<string> (loc_[XPath("PRECISEPOLYGON/@ref")]));

	vector<pair<double, double> > poly;
	for (XPathIterator iter = XPath("PRECISEPOLYGON/POINT2D").evaluate(loc_); iter; ++iter) {

		double x = extract<double> ((*iter)[XPath("@x")]);
		double y = extract<double> ((*iter)[XPath("@y")]);
		poly.push_back(pair<double, double> (x, y));
	}

	object.setPolygon(poly, MILLIMETER);
}

vector<pair<double, double> > ObjectPositionData::Hypothesis::scalePolygon(
		vector<pair<double, double> > poly, double factor) const {
	for (vector<pair<double, double> >::iterator it = poly.begin(); it
			!= poly.end(); ++it) {
		it->first *= factor;
		it->second *= factor;
	}
	return poly;
}

vector<pair<double, double> > ObjectPositionData::Hypothesis::getPolygon(
		ObjectPositionData::Unit unit) const {
	switch (unit) {
	case METER:
		return scalePolygon(polygon, 0.001);
	case CENTIMETER:
		return scalePolygon(polygon, 0.1);
	case MILLIMETER:
	default:
		return polygon;
	}
}
void ObjectPositionData::Hypothesis::setPolygon(const vector<pair<double,
		double> > &polygon, Unit unit) {
	switch (unit) {
	case METER:
		this->polygon = scalePolygon(polygon, 1000);
		break;
	case CENTIMETER:
		this->polygon = scalePolygon(polygon, 10);
		break;
	case MILLIMETER:
	default:
		this->polygon = polygon;
		break;
	}
}

string ObjectPositionData::Hypothesis::getCoordinateKind() const {
	return coordinateKind;
}

void ObjectPositionData::Hypothesis::setCoordinateKind(const string &kind) {
	coordinateKind = kind;
}

string ObjectPositionData::Hypothesis::getReference() const {
	return reference;
}

void ObjectPositionData::Hypothesis::setReference(const string &ref) {
	reference = ref;
}

bool ObjectPositionData::operator==(const ObjectPositionData &other) const {

	bool isHyp = true;

	try {
		vector<Hypothesis*> h0 = getPositionHypotheses();
		vector<Hypothesis*> h1 = other.getPositionHypotheses();

		if (h0.size() != h1.size()) {
			return false;
		}

		vector<Hypothesis*>::iterator it0 = h0.begin();
		for (vector<Hypothesis*>::iterator it1 = h1.begin(); it1 != h1.end(); it1++) {

			Hypothesis hyp0 = **it0;
			Hypothesis hyp1 = **it1;

			if (!(hyp0 == hyp1)) {
				isHyp = false;
			}
			it0++;
		}

		return ObjectData::operator==(other) && isHyp;
	} catch (bad_cast e) {
		return false;
	}

}

bool ObjectPositionData::Hypothesis::operator==(const Hypothesis &other) const {

	bool isPoly = true;
	if (other.getPolygon(MILLIMETER).size() != getPolygon(MILLIMETER).size()) {
		isPoly = false;
	} else {
		for (int i = 0; i < (int) getPolygon(MILLIMETER).size(); i++) {
			if (getPolygon(MILLIMETER)[i].first
					!= other.getPolygon(MILLIMETER)[i].first || getPolygon(
					MILLIMETER)[i].second
					!= other.getPolygon(MILLIMETER)[i].second || getReference()
					!= other.getReference() || getCoordinateKind()
					!= other.getCoordinateKind()) {
				isPoly = false;
			}
		}
	}
	return ObjectData::Hypothesis::operator==(other) && isPoly;

}

vector<ObjectPositionData::Hypothesis*> ObjectPositionData::getPositionHypotheses() const {

	vector<ObjectData::Hypothesis*> h0 = ObjectData::getHypotheses();
	vector<Hypothesis*> h1;

	for (vector<ObjectData::Hypothesis*>::iterator it = h0.begin(); it
			!= h0.end(); it++) {
		try {
			Hypothesis* hyp = dynamic_cast<Hypothesis*> (*it);
			h1.push_back(hyp);
		} catch (bad_cast e) {
			continue;
		}
	}
	return h1;
}
