#ifndef OBJECTPOSITIONDATA_H_
#define OBJECTPOSITIONDATA_H_

#include <string>
#include <vector>
#include <list>

#include <xmltio/xmltio.hpp>

#include "Type.h"
#include "ObjectData.h"

namespace airobots {

/**
 * Results of the object recognition.
 *
 * @author lziegler
 */
class ObjectPositionData: public ObjectData {

public:
	enum Unit {
		MILLIMETER, CENTIMETER, METER
	};

	class Hypothesis: public ObjectData::Hypothesis {
	private:
		std::vector<std::pair<double, double> > polygon;
		std::string reference;
		std::string coordinateKind;
		std::vector<std::pair<double, double> > scalePolygon(std::vector<
				std::pair<double, double> > poly, double factor) const;

	public:

		static std::string getBaseTag();
		static std::string getTemplate();

		Hypothesis();
		Hypothesis(std::string generator);

		std::vector<std::pair<double, double> > getPolygon(Unit unit) const;
		void setPolygon(const std::vector<std::pair<double, double> > &polygon,
				Unit unit);

		std::string getCoordinateKind() const;
		void setCoordinateKind(const std::string &kind);

		std::string getReference() const;
		void setReference(const std::string &ref);

		virtual bool operator==(const Hypothesis &other) const;

		void sanitizeLocation(xmltio::Location&) const;
		void fillInto(xmltio::Location &loc) const;
		static void fromLocation(const xmltio::Location &loc, Hypothesis &type);

	};

	static std::string getBaseTag();

	static std::string getTemplate();

	ObjectPositionData();
	ObjectPositionData(std::string generator);

	std::vector<Hypothesis*> getPositionHypotheses() const;

	virtual bool operator==(const ObjectPositionData &other) const;

	void sanitizeLocation(xmltio::Location&) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc,
			ObjectPositionData &type);

};

}

#endif /* OBJECTPOSITIONDATA_H_ */
