/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */

#include <cmath>

#include "PersonData.h"

using namespace std;
using namespace airobots;
using namespace xmltio;

std::string PersonData::getBaseTag() {
	return "PERSON";
}

std::string PersonData::getTemplate() {
	return std::string(
			"<" + getBaseTag() + ">"
					+ "<STATES walking=\"\" facing=\"\" gazing=\"\" talking=\"\" dist=\"\" ang=\"\" poi=\"\" />"
					+ "<IDENT name=\"none\" id=\"\"/>"
					+ "<NEIGHBOUR dist=\"-1\" id=\"-1\"/>"
					+ "<GENERATOR/>") + Timestamp::getTemplate() + "</"
			+ getBaseTag() + ">";
}

PersonData::PersonData(std::string generator) :
	Type(generator) {

}

PersonData::PersonData() :
	Type("") {

}

float PersonData::getAngle() const {
	return angle;
}

float PersonData::getDistance() const {
	return distance;
}

bool PersonData::isFacing() const {
	return facing;
}

float PersonData::getGaze() const {
	return gaze;
}

int PersonData::getIdentId() const {
	return identId;
}

std::string PersonData::getIdentName() const {
	return identName;
}

bool PersonData::hasNeighbour() const {
	return neighbour;
}

float PersonData::getNeighbourDistance() const {
	return neighbourDistance;
}

int PersonData::getNeighbourId() const {
	return neighbourId;
}

bool PersonData::isPoi() const {
	return poi;
}

bool PersonData::isTalking() const {
	return talking;
}

bool PersonData::isWalking() const {
	return walking;
}

boost::shared_ptr<Face> PersonData::getFace() const {
	return face;
}

bool PersonData::hasFace() const {
	return face.get();
}

boost::shared_ptr<Legs> PersonData::getLegs() const {
	return legs;
}

bool PersonData::hasLegs() const {
	return legs.get();
}

boost::shared_ptr<Shirt> PersonData::getShirt() const {
	return shirt;
}

bool PersonData::hasShirt() const {
	return shirt.get();
}

void PersonData::setAngle(float angle) {
	this->angle = angle;
}

void PersonData::setDistance(float distance) {
	this->distance = distance;
}

void PersonData::setFacing(bool facing) {
	this->facing = facing;
}

void PersonData::setGaze(float gaze) {
	this->gaze = gaze;
}

void PersonData::setIdentId(int identId) {
	this->identId = identId;
}

void PersonData::setIdentName(std::string identName) {
	this->identName = identName;
}

void PersonData::setHasNeighbour(bool hasNeighbour) {
	this->neighbour = hasNeighbour;
}

void PersonData::setNeighbourDistance(float neighbourDistance) {
	this->neighbourDistance = neighbourDistance;
}

void PersonData::setNeighbourId(int neighbourId) {
	this->neighbourId = neighbourId;
}

void PersonData::setPoi(bool poi) {
	this->poi = poi;
}

void PersonData::setTalking(bool talking) {
	this->talking = talking;
}

void PersonData::setWalking(bool walking) {
	this->walking = walking;
}

void PersonData::setFace(boost::shared_ptr<Face> face) {
	this->face = face;
}

void PersonData::setLegs(boost::shared_ptr<Legs> legs) {
	this->legs = legs;
}

void PersonData::setShirt(boost::shared_ptr<Shirt> shirt) {
	this->shirt = shirt;
}

void PersonData::sanitizeLocation(xmltio::Location &loc) const {
	
}

void PersonData::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	loc[XPath("STATES/@walking")] = (isWalking() ? "YES" : "NO");
	loc[XPath("STATES/@facing")] = (isFacing() ? "YES" : "NO");
	loc[XPath("STATES/@gazing")] = getGaze() * 180.0 / M_PI;
	loc[XPath("STATES/@talking")] = (isTalking() ? "YES" : "NO");
	loc[XPath("STATES/@dist")] = getDistance();
	loc[XPath("STATES/@ang")] = getAngle() * 180.0 / M_PI;
	loc[XPath("STATES/@poi")] = (isPoi() ? "YES" : "NO");
	if (getIdentName() == "") {
		loc[XPath("IDENT/@name")] = "identifying";
	} else {
		loc[XPath("IDENT/@name")] = getIdentName();
	}
	loc[XPath("IDENT/@id")] = getIdentId();
	loc[XPath("NEIGHBOUR/@dist")] = getNeighbourDistance();
	loc[XPath("NEIGHBOUR/@id")] = getNeighbourId();

	if (hasFace()) {
		Location faceLoc(Face::getTemplate(), Face::getBaseTag());
		faceLoc = *(getFace());
		loc.add(faceLoc);
	}

	if (hasLegs()) {
		Location legLoc(Legs::getTemplate(), Legs::getBaseTag());
		legLoc = *(getLegs());
		loc.add(legLoc);
	}

	if (hasShirt()) {
		Location shirtLoc(Shirt::getTemplate(), Shirt::getBaseTag());
		shirtLoc = *(getShirt());
		loc.add(shirtLoc);
	}

}

void PersonData::fromLocation(const xmltio::Location &loc_, PersonData &pd) {

	string strHelpString;

	Type::fromLocation(loc_, pd);

	strHelpString = extract<string> (loc_[XPath("STATES/@walking")]);
	pd.setWalking(strcmp("NO", strHelpString.c_str()));
	strHelpString = extract<string> (loc_[XPath("STATES/@facing")]);
	pd.setFacing(strcmp("NO", strHelpString.c_str()));
	pd.setGaze(extract<float> (loc_[XPath("STATES/@gazing")]) * M_PI / 180.0);
	strHelpString = extract<string> (loc_[XPath("STATES/@talking")]);
	pd.setTalking(strcmp("NO", strHelpString.c_str()));
	pd.setDistance(extract<float> (loc_[XPath("STATES/@dist")]));
	pd.setAngle(extract<float> (loc_[XPath("STATES/@ang")]) * M_PI / 180.0);
	strHelpString = extract<string> (loc_[XPath("STATES/@poi")]);
	pd.setPoi(strcmp("NO", strHelpString.c_str()));
	pd.setIdentName(extract<string> (loc_[XPath("IDENT/@name")]));
	pd.setIdentId(extract<int> (loc_[XPath("IDENT/@id")]));
	pd.setNeighbourDistance(extract<float> (loc_[XPath("NEIGHBOUR/@dist")]));
	pd.setNeighbourId(extract<int> (loc_[XPath("NEIGHBOUR/@id")]));
	// TODO this code could be placed in the getter
	pd.setHasNeighbour(((pd.getNeighbourId() == -1) ? false : true));

	try {
		pd.setFace(boost::shared_ptr<Face>(new Face(extract<Face> (loc_[XPath(
				Face::getBaseTag())]))));
		assert(pd.hasFace());
	} catch (xmltio::XMLTIO_Exception & e) {
	}

	try {
		pd.setLegs(boost::shared_ptr<Legs>(new Legs(extract<Legs> (loc_[XPath(
				Legs::getBaseTag())]))));
		assert(pd.hasLegs());
	} catch (xmltio::XMLTIO_Exception & e) {
	}

	try {
		pd.setShirt(boost::shared_ptr<Shirt>(new Shirt(extract<Shirt> (loc_[XPath(
				Shirt::getBaseTag())]))));
		assert(pd.hasShirt());
	} catch (xmltio::XMLTIO_Exception & e) {
	}

}
