 /** ==========================================================================
 * 
 * @file		PersonDataList.cpp --
 * @brief		Source file for the list of person data.
 *
 * @author	Denis Schulze <dschulze@techfak.uni-bielefeld.de>
 * 
 * ========================================================================== 
 **/

#include "PersonDataList.h"

using namespace airobots;

PersonDataList::PersonDataList() {

}

PersonDataList::~PersonDataList() {

}

std::string 
PersonDataList::getBaseTag() {
	return "PERSONS";
}

std::string 
PersonDataList::getTemplate() {
	return "<" + getBaseTag() + ">" + "<GENERATOR/>"
		+ Timestamp::getTemplate() + "</" + getBaseTag() + ">";
}

void 
PersonDataList::addPersonData(airobots::PersonData data) {
	personData.push_back(data);
}

std::vector<PersonData>
PersonDataList::getPersonData() {
	return personData;
}

int
PersonDataList::size() {
	return personData.size();
}

void
PersonDataList::sanitizeLocation(xmltio::Location &loc) const {

	xmltio::XPath xp(PersonData::getBaseTag());
	while (xmltio::XPathIterator i = xp.evaluate(loc)) {
		xmltio::Location l(*i);
		loc.remove(l);
	}

}

void 
PersonDataList::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	int nrPersons = personData.size();
	for (int i = 0; i < nrPersons; i++) {
		xmltio::Location personLoc(airobots::PersonData::getTemplate(), airobots::PersonData::getBaseTag());
		personLoc = personData[i];
		loc.add(personLoc);		
	}

}

void 
PersonDataList::fromLocation(const xmltio::Location &loc, PersonDataList &list) {	

	Type::fromLocation(loc, list);

	for (xmltio::XPathIterator i = xmltio::XPath(airobots::PersonData::getBaseTag()).evaluate(loc); i; ++i) {
		xmltio::Location personLoc(*i);
		PersonData data = xmltio::extract<airobots::PersonData>(personLoc);
		list.addPersonData(data);
	}

}
