/*
 * Point2D.cpp
 *
 *  Created on: Sep 6, 2010
 *      Author: maschroe
 */

#include "Point2D.h"

using namespace airobots;
using namespace xmltio;
using namespace std;

Point2D::Point2D() :
	scope(GLOBAL) {
}

Point2D::Point2D(float x, float y, LengthUnit unit) :
	x(x), y(y), scope(GLOBAL) {
	setX(x, unit);
	setY(y, unit);
}
Point2D::Point2D(float x, float y, LengthUnit unit, Point2D::Scope scope) :
	x(x), y(y), scope(scope) {
	setX(x, unit);
	setY(y, unit);
}

Point2D::~Point2D() {
}

std::string Point2D::getBaseTag() {
	return "POINT2D";
}

std::string Point2D::getTemplate() {
	return "<" + getBaseTag() + " x=\"\" y=\"\" scope=\"\" />";
}

Point2D::Scope Point2D::getScope() const {
	return scope;
}

float Point2D::getX(LengthUnit unit) const {
	return UnitConverter::convert(x, MILLIMETER, unit);
}

float Point2D::getY(LengthUnit unit) const {
	return UnitConverter::convert(y, MILLIMETER, unit);
}

void Point2D::setScope(Scope scope) {
	this->scope = scope;
}

void Point2D::setX(float x, LengthUnit unit) {
	this->x = UnitConverter::convert(x, unit, MILLIMETER);
}

void Point2D::setY(float y, LengthUnit unit) {
	this->y = UnitConverter::convert(y, unit, MILLIMETER);
}

void Point2D::fillInto(xmltio::Location &loc) const {

	loc[XPath("@x")] = getX(MILLIMETER);
	loc[XPath("@y")] = getY(MILLIMETER);

	if (scope == LOCAL) {
		loc[XPath("@scope")] = "LOCAL";
	} else {
		loc[XPath("@scope")] = "GLOBAL";
	}
}

void Point2D::fromLocation(const xmltio::Location &loc_, Point2D &object) {

	object.setX(extract<float> (loc_[XPath("@x")]), MILLIMETER);
	object.setY(extract<float> (loc_[XPath("@y")]), MILLIMETER);

	string scopeStr = extract<string> (loc_[XPath("@scope")]);
	if (scopeStr == "LOCAL") {
		object.setScope(LOCAL);
	} else {
		object.setScope(GLOBAL);
	}
}

void airobots::Point2D::sanitizeLocation(xmltio::Location & loc) const
{
	xmltio::XPath xp(getBaseTag());
	while (xmltio::XPathIterator i = xp.evaluate(loc)) {
		xmltio::Location l(*i);
		loc.remove(l);
	}
}


