/*
 * Point3D.cpp
 *
 *  Created on: Jun 4, 2010
 *      Author: lziegler
 */

#include "Point3D.h"

using namespace airobots;
using namespace xmltio;
using namespace std;

Point3D::Point3D() :
	scope(GLOBAL) {
}

Point3D::Point3D(float x, float y, float z, LengthUnit unit) :
	x(x), y(y), z(z), scope(GLOBAL) {
	setX(x, unit);
	setY(y, unit);
	setZ(z, unit);
}
Point3D::Point3D(float x, float y, float z, LengthUnit unit, Point3D::Scope scope) :
	x(x), y(y), z(z), scope(scope) {
	setX(x, unit);
	setY(y, unit);
	setZ(z, unit);
}

Point3D::~Point3D() {
}

std::string Point3D::getBaseTag() {
	return "POINT3D";
}

std::string Point3D::getTemplate() {
	return "<" + getBaseTag() + " x=\"\" y=\"\" z=\"\" scope=\"\" />";
}

Point3D::Scope Point3D::getScope() const {
	return scope;
}

float Point3D::getX(LengthUnit unit) const {
	return UnitConverter::convert(x, MILLIMETER, unit);
}

float Point3D::getY(LengthUnit unit) const {
	return UnitConverter::convert(y, MILLIMETER, unit);
}

float Point3D::getZ(LengthUnit unit) const {
	return UnitConverter::convert(z, MILLIMETER, unit);
}

void Point3D::setScope(Scope scope) {
	this->scope = scope;
}

void Point3D::setX(float x, LengthUnit unit) {
	this->x = UnitConverter::convert(x, unit, MILLIMETER);
}

void Point3D::setY(float y, LengthUnit unit) {
	this->y = UnitConverter::convert(y, unit, MILLIMETER);
}

void Point3D::setZ(float z, LengthUnit unit) {
	this->z = UnitConverter::convert(z, unit, MILLIMETER);
}

void Point3D::fillInto(xmltio::Location &loc) const {

	loc[XPath("@x")] = getX(MILLIMETER);
	loc[XPath("@y")] = getY(MILLIMETER);
	loc[XPath("@z")] = getZ(MILLIMETER);

	if (scope == LOCAL) {
		loc[XPath("@scope")] = "LOCAL";
	} else {
		loc[XPath("@scope")] = "GLOBAL";
	}
}

void Point3D::fromLocation(const xmltio::Location &loc_, Point3D &object) {

	object.setX(extract<float> (loc_[XPath("@x")]), MILLIMETER);
	object.setY(extract<float> (loc_[XPath("@y")]), MILLIMETER);
	object.setZ(extract<float> (loc_[XPath("@z")]), MILLIMETER);

	string scopeStr = extract<string> (loc_[XPath("@scope")]);
	if (scopeStr == "LOCAL") {
		object.setScope(LOCAL);
	} else {
		object.setScope(GLOBAL);
	}
}

void airobots::Point3D::sanitizeLocation(xmltio::Location & loc) const
{
	xmltio::XPath xp(getBaseTag());
	while (xmltio::XPathIterator i = xp.evaluate(loc)) {
		xmltio::Location l(*i);
		loc.remove(l);
	}
}


