/*
 * PositionData.h
 *
 *  Created on: Jul 31, 2009
 *      Author: jwienke
 */

#ifndef POSITIONDATA_H_
#define POSITIONDATA_H_

#include <string>
#include <xmltio/xmltio.hpp>
#include "Type.h"

namespace airobots {

/**
 * Base class for types that contain positions.
 * Not abstract anymore.
 */
class PositionData : public Type{

public:

	static std::string getBaseTag();

	static std::string getTemplate();

	PositionData();
	PositionData(float x, float y, float theta);
	virtual ~PositionData();

	float getX() const;
	void setX(float x);

	float getY() const;
	void setY(float y);

	float getTheta() const;
	void setTheta(float theta);

	void sanitizeLocation(xmltio::Location &loc) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, PositionData &pos);

private:

	/**
	 * x coordinate relative to starting position [m],
	 * at orientation = 0, x axis is in forward direction
	 */
	float x; // m

	/**
	 * y coordinate relative to starting position [m],
	 * perpendicular-left to the positive x axis
	 */
	float y; // m

	/**
	 * orientation relative to starting orientation [rad],
	 * range is [-PI .. +PI], counter-clockwise
	 */
	float theta; // rad

};

}

#endif /* POSITIONDATA_H_ */
