#include "RobotState.h"

using namespace std;
using namespace airobots;
using namespace xmltio;

const string RobotState::CURRENT_STATE_ELEMENT_NAME = "CURRENTSTATE";
const string RobotState::PREVIOUS_STATE_ELEMENT_NAME = "PREVIOUSSTATE";
const string RobotState::STATE_TARGET_ELEMENT_NAME = "STATETARGET";

RobotState::RobotState() {
}

RobotState::~RobotState() {
}

std::string RobotState::getBaseTag()
{
	return "ROBOTSTATE";
}



void RobotState::fillInto(xmltio::Location & loc) const
{
	Type::fillInto(loc);

	switch (getCurrentState()) {
	case DRIVE_TO:
		loc[XPath(CURRENT_STATE_ELEMENT_NAME)] = "DRIVE_TO";
		break;
	case TALK_TO:
		loc[XPath(CURRENT_STATE_ELEMENT_NAME)] = "TALK_TO";
		break;
	case GO_HOME:
		loc[XPath(CURRENT_STATE_ELEMENT_NAME)] = "GO_HOME";
		break;
	case WAITING:
		loc[XPath(CURRENT_STATE_ELEMENT_NAME)] = "WAITING";
		break;
	default:
		break;
	}

	switch (getPreviousState()) {
	case DRIVE_TO:
		loc[XPath(PREVIOUS_STATE_ELEMENT_NAME)] = "DRIVE_TO";
		break;
	case TALK_TO:
		loc[XPath(PREVIOUS_STATE_ELEMENT_NAME)] = "TALK_TO";
		break;
	case GO_HOME:
		loc[XPath(PREVIOUS_STATE_ELEMENT_NAME)] = "GO_HOME";
		break;
	case WAITING:
		loc[XPath(PREVIOUS_STATE_ELEMENT_NAME)] = "WAITING";
		break;
	default:
		break;
	}

	loc[XPath(STATE_TARGET_ELEMENT_NAME)] = getTargetName();
}



std::string RobotState::getTemplate()
{
	return "<" + getBaseTag() + ">"
			+ "<GENERATOR/>"
			+ Timestamp::getTemplate()
			+ "<" + CURRENT_STATE_ELEMENT_NAME + "/>"
			+ "<" + PREVIOUS_STATE_ELEMENT_NAME + "/>"
			+ "<" + STATE_TARGET_ELEMENT_NAME + "/>"
			+ "</" + getBaseTag() + ">";
}



void RobotState::sanitizeLocation(xmltio::Location & loc) const
{
}



void RobotState::fromLocation(const xmltio::Location & loc, RobotState & rs)
{
	Type::fromLocation(loc, rs);

	string pstateString = extract<string>(loc[XPath(PREVIOUS_STATE_ELEMENT_NAME)]);
	string cstateString = extract<string>(loc[XPath(CURRENT_STATE_ELEMENT_NAME)]);
	string target = extract<string>(loc[XPath(STATE_TARGET_ELEMENT_NAME)]);

	RobotStates pstate;

	if (pstateString.compare("DRIVE_TO") == 0) {
		pstate = DRIVE_TO;
	} else if (pstateString.compare("TALK_TO") == 0) {
		pstate = TALK_TO;
	} else if (pstateString.compare("GO_HOME") == 0) {
		pstate = GO_HOME;
	} else if (pstateString.compare("WAITING") == 0) {
		pstate = WAITING;
	}

	RobotStates cstate;

	if (cstateString.compare("DRIVE_TO") == 0) {
		cstate = DRIVE_TO;
	} else if (cstateString.compare("TALK_TO") == 0) {
		cstate = TALK_TO;
	} else if (cstateString.compare("GO_HOME") == 0) {
		cstate = GO_HOME;
	} else if (cstateString.compare("WAITING") == 0) {
		cstate = WAITING;
	}

	rs.setPreviousState(pstate);
	rs.setCurrentState(cstate);
	rs.setTargetName(target);
}

RobotState::RobotStates RobotState::getPreviousState() const
{
	return previousState;
}



void RobotState::setPreviousState(RobotStates state)
{
	previousState = state;
}



void RobotState::setCurrentState(RobotStates state)
{
	currentState = state;
}



RobotState::RobotStates RobotState::getCurrentState() const
{
	return currentState;
}



std::string RobotState::getTargetName() const
{
	return targetName;
}



bool airobots::RobotState::operator ==(const RobotState & other) const
{
	return currentState == other.getCurrentState()
		&& previousState == other.getPreviousState()
		&& targetName.compare(other.getTargetName()) == 0;
}

void RobotState::setTargetName(std::string target)
{
	targetName = target;
}




