#ifndef ROBOTSTATE_H_
#define ROBOTSTATE_H_

#include "Type.h"

namespace airobots {

class RobotState: public Type {
public:

	enum RobotStates {
        /**
         * The robot is driving to a position
         */
        DRIVE_TO,
        /**
         * The robot shall speak to an interaction partner
         */
        TALK_TO,
        /**
         * The robot shall return to the HOME position, normally (0,0)
         */
        GO_HOME,
        /**
         * The robot is waiting for command somewhere
         */
        WAITING
	};

	RobotState();
	virtual ~RobotState();

	static std::string getBaseTag();
	static std::string getTemplate();

	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, RobotState & rs);
	void sanitizeLocation(xmltio::Location &loc) const;

	virtual bool operator==(const RobotState &other) const;

	RobotStates getPreviousState() const;
	RobotStates getCurrentState() const;
	std::string getTargetName() const;

	void setPreviousState(RobotStates state);
	void setCurrentState(RobotStates state);
	void setTargetName(std::string target);

private:

	static const std::string CURRENT_STATE_ELEMENT_NAME;
	static const std::string PREVIOUS_STATE_ELEMENT_NAME;
	static const std::string STATE_TARGET_ELEMENT_NAME;

	RobotStates previousState;
	RobotStates currentState;
	std::string targetName;
};

}

#endif /* ROBOTSTATE_H_ */
