#include "SRCalibratedData.h"
#include "Point3D.h"

using namespace xmltio;
using namespace airobots;
using namespace std;

std::string SRCalibratedData::getBaseTag() {
	return "SRCALIBRATEDDATA";
}

std::string SRCalibratedData::getTemplate() {
	return "<" + getBaseTag() + " scope=\"\"><GENERATOR/>" + Timestamp::getTemplate()+ "</" + getBaseTag() + ">";
}

SRCalibratedData::SRCalibratedData() :
	Type(), scope(GLOBAL) {
}

SRCalibratedData::SRCalibratedData(std::string generator) :
	Type(generator), scope(GLOBAL) {
}

SRCalibratedData::SRCalibratedData(std::string generator,
		SRCalibratedData::Scope scope) :
	Type(generator), scope(scope) {
}

void SRCalibratedData::setScope(SRCalibratedData::Scope scope) {
	this->scope = scope;
}
SRCalibratedData::Scope SRCalibratedData::getScope() const {
	return scope;
}

void SRCalibratedData::sanitizeLocation(xmltio::Location &loc) const {
	XPath xp(Point3D::getBaseTag());
	while (XPathIterator i = xp.evaluate(loc)) {
		Location l(*i);
		loc.remove(l);
	}
}

void SRCalibratedData::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	if (scope == LOCAL) {
		loc[XPath("@scope")] = "LOCAL";
	} else {
		loc[XPath("@scope")] = "GLOBAL";
	}
	for (vector<Point3D>::const_iterator it = this->begin(); it != this->end(); it++) {

		Location obj(Point3D::getTemplate(), XPath(Point3D::getBaseTag()));

		it->fillInto(obj);
		loc.add(obj);
	}
}

void SRCalibratedData::fromLocation(const xmltio::Location &loc_,
		SRCalibratedData &object) {

	string scopeStr = extract<string> (loc_[XPath("@scope")]);
	if (scopeStr == "LOCAL") {
		object.setScope(LOCAL);
	} else {
		object.setScope(GLOBAL);
	}
	for (XPathIterator iter = XPath(Point3D::getBaseTag()).evaluate(loc_); iter; ++iter) {
		Location l(*iter);
		Point3D data = extract<Point3D> (l);
		object.push_back(data);
	}
}
