#ifndef SRCALIBRATEDDATA_H_
#define SRCALIBRATEDDATA_H_

#include <string>
#include <vector>
#include <list>

#include <xmltio/xmltio.hpp>

#include "Type.h"
#include "Point3D.h"

namespace airobots {

/**
 * SwissRanger calibrated 3d data.
 * The points are positioned by rows. There is one Point3D object for each
 * pixel of the SwissRanger camera. Access to a specific row and col schould be
 * done like this:
 *
 * index = row * width + col
 *
 * @author lziegler
 */
class SRCalibratedData: public std::vector<Point3D>, public Type {

public:

	enum Scope {
		GLOBAL, LOCAL
	};

	static std::string getBaseTag();
	static std::string getTemplate();

	SRCalibratedData();
	SRCalibratedData(std::string generator);
	SRCalibratedData(std::string generator, SRCalibratedData::Scope scope);

	void setScope(SRCalibratedData::Scope scope);
	SRCalibratedData::Scope getScope() const;

	void sanitizeLocation(xmltio::Location&) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc,
			SRCalibratedData &type);

private:
	SRCalibratedData::Scope scope;

};

}

#endif /* SRCALIBRATEDDATA_H_ */
