/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; c-file-offsets: ((substatement-open . 0)) -*- */
/**
 * @file     speedrefdata.cc
 * @author   Thorsten Spexard
 * @date     08.08.2005
 * @brief    Converting and extracting speedref data to and from xml-structure
 */

#include "SpeedData.h"

using namespace std;
using namespace xmltio;
using namespace airobots;

std::string SpeedData::getBaseTag() {
	return "SPEEDS";
}

std::string SpeedData::getTemplate() {
	return std::string("<") + getBaseTag() + ">" + "<GENERATOR/>"
			+ Timestamp::getTemplate() + MicroTimestamp::getTemplate()
			+ "<ROTATIONSPEED quant=\"\"/>"
			+ "<TRANSLATIONSPEED quant=\"\"/>" + "</" + getBaseTag() + ">";
}

SpeedData::SpeedData() :
	Type("unknown") {

}

double SpeedData::getTranslation() const {
	return translation;
}

void SpeedData::setTranslation(double translation) {
	this->translation = translation;
}

double SpeedData::getRotation() const {
	return rotation;
}

void SpeedData::setRotation(double rotation) {
	this->rotation = rotation;
}

void SpeedData::sanitizeLocation(xmltio::Location &loc) const {
	
}

void SpeedData::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	loc[XPath(MicroTimestamp::getBaseTag())] = getReadTime();
	loc[XPath("ROTATIONSPEED/@quant")] = getRotation();
	loc[XPath("TRANSLATIONSPEED/@quant")] = getTranslation();

}

void SpeedData::fromLocation(const xmltio::Location &loc_, SpeedData &sd) {

	Type::fromLocation(loc_, sd);

	sd.setRotation(extract<double> (loc_[XPath("ROTATIONSPEED/@quant")]));
	sd.setTranslation(extract<double> (loc_[XPath("TRANSLATIONSPEED/@quant")]));

	sd.setReadTime(extract<MicroTimestamp> (loc_[XPath(
			MicroTimestamp::getBaseTag())]));

}
