/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; c-file-offsets: ((substatement-open . 0)) -*- */
/**
 * @file     speedrefdata.h
 * @author   Thorsten Spexard
 * @date     08.08.2005
 * @brief    Converting and extracting speedref data to and from xml-structure
 */

#ifndef SPEEDREFDATA_CPP
#define SPEEDREFDATA_CPP

#include <string>

#include <xmltio/xmltio.hpp>

#include "Type.h"
#include "Ticked.h"

/**
 * xml structure for speedref data
 */

namespace airobots {

/**
 * C++ structure which is converted to / extracted from xml-data
 */
class SpeedData: public Type, public Ticked {

public:

	static std::string getBaseTag();

	static std::string getTemplate();

	SpeedData();

	/**
	 * Returns the translational speed in m/s.
	 */
	double getTranslation() const;
	void setTranslation(double translation);

	/**
	 * Returns the rotational speed in rad/s.
	 */
	double getRotation() const;
	void setRotation(double rotation);

	void sanitizeLocation(xmltio::Location &loc) const;
	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, SpeedData &type);

private:

	/**
	 * absolute translation velocity in meters/s
	 */
	double translation;

	/**
	 * absolute rotation velocity in rad/s
	 */
	double rotation;

};

}

#endif
