/** ==========================================================================
 * 
 * @file		SplocAngleData.cpp --
 * @brief		Source file for the angle output of the sploc application.
 *
 * @author	Denis Schulze <dschulze@techfak.uni-bielefeld.de>
 * @author  dklotz
 * 
 * ========================================================================== 
 **/

#include "SplocAngleData.h"

using namespace airobots;
using namespace xmltio;

SplocAngleData::SplocAngleData() {

}

SplocAngleData::~SplocAngleData() {

}

std::string SplocAngleData::getBaseTag() {
	return "SPLOCANGLES";
}

std::string SplocAngleData::getTemplate() {

	std::stringstream tmp;
	tmp << "<" << getBaseTag() << ">" << "<GENERATOR/>"
			<< Timestamp::getTemplate()
			<< MicroTimestamp::getTemplate()
			<< "<ANGLES/>" << "</" << getBaseTag()
			<< ">";

	return tmp.str();

}

std::vector<float> SplocAngleData::getAngles() {
	return angles;
}

void SplocAngleData::setAngles(std::vector<float> angles) {
	this->angles = angles;
}

void SplocAngleData::sanitizeLocation(xmltio::Location &loc) const {

}

void SplocAngleData::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	loc[XPath(MicroTimestamp::getBaseTag())] = getReadTime();

	Location anglesLoc = loc[XPath("ANGLES")];

	for (unsigned int i = 0; i < angles.size(); i++) {
		Location angleLoc("<ANGLE value=\"\"/>", "/ANGLE");

		angleLoc[XPath("@value")] = angles[i];

		anglesLoc.add(angleLoc);
	}

}

void SplocAngleData::fromLocation(const xmltio::Location &loc,
		SplocAngleData &data) {

	Type::fromLocation(loc, data);

	std::vector<float> angles;

	for (XPathIterator it = XPath("ANGLES/ANGLE").evaluate(loc); it; ++it) {
		angles.push_back(xmltio::extract<float>((*it)[XPath("@value")]));
	}

	data.setAngles(angles);

	data.setReadTime(extract<MicroTimestamp> (loc[XPath(
				MicroTimestamp::getBaseTag())]));
}
