#include "StateConfiguration.h"

using namespace std;
using namespace airobots;
using namespace xmltio;

const string StateConfiguration::STATE_ELEMENT = "STATE";
const string StateConfiguration::SYSTEM_STATES_ELEMENT = "SYSTEMSTATES";
const string StateConfiguration::SYSTEM_NAME_ELEMENT = "SYSTEMNAME";

StateConfiguration::StateConfiguration() {
	systemName = "unknown";
	states = set<string>();
}

StateConfiguration::~StateConfiguration() {
}

void airobots::StateConfiguration::addStates(std::set<std::string> stateNames)
{
	states.insert(stateNames.begin(), stateNames.end());
}



const std::string airobots::StateConfiguration::getSystemName() const
{
	return systemName;
}



std::string airobots::StateConfiguration::getBaseTag()
{
	return "STATEVIZCONFIG";
}



std::string airobots::StateConfiguration::getTemplate()
{
	return "<" + getBaseTag() + ">"
		+ "<GENERATOR/>"
		+ Timestamp::getTemplate()
		+ "<" + SYSTEM_NAME_ELEMENT + "/>"
		+ "<" + SYSTEM_STATES_ELEMENT + ">"
		+ "</" + SYSTEM_STATES_ELEMENT + ">"
		+ "</" + getBaseTag() + ">";
}



void airobots::StateConfiguration::addState(std::string stateName)
{
	states.insert(stateName);
}



void airobots::StateConfiguration::setSystemName(std::string name)
{
	systemName = name;
}



void airobots::StateConfiguration::fromLocation(const xmltio::Location & loc, StateConfiguration & sc)
{
	Type::fromLocation(loc, sc);

	string name = extract<string>(loc[XPath(SYSTEM_NAME_ELEMENT)]);
	sc.setSystemName(name);

	Location statesLoc = loc[XPath(SYSTEM_STATES_ELEMENT)];
	for (XPathIterator i = XPath(STATE_ELEMENT).evaluate(statesLoc); i; ++i) {
		Location stateLoc(*i);
		string value = extract<string>(stateLoc);
		sc.addState(value);
	}
}



void airobots::StateConfiguration::removeAllStates()
{
	states.clear();
}



void airobots::StateConfiguration::sanitizeLocation(xmltio::Location & loc) const
{
}



void airobots::StateConfiguration::fillInto(xmltio::Location & loc) const
{
	Type::fillInto(loc);

	loc[XPath(SYSTEM_NAME_ELEMENT)] = getSystemName();

	set<string>::iterator i = states.begin();
	for (; i != states.end(); ++i) {
		Location stateLoc("<" + STATE_ELEMENT + "/>", STATE_ELEMENT);
		stateLoc = *i;
		loc[XPath(SYSTEM_STATES_ELEMENT)].add(stateLoc);
	}
}



std::set<std::string> airobots::StateConfiguration::getStates()
{
	return states;
}


