/*
 * Timestamp.cpp
 *
 *  Created on: Jul 22, 2009
 *      Author: jwienke
 */

#include "Timestamp.h"

#include <sstream>
#include <cmath>

#include <sys/time.h>
#include <time.h>

using namespace airobots;
using namespace xmltio;
using namespace std;

std::string Timestamp::getBaseTag() {
	return "TIMESTAMP";
}

std::string Timestamp::getTemplate() {
	return "<" + getBaseTag()
			+ "><INSERTED value=\"\"/><UPDATED value=\"\"/></" + getBaseTag()
			+ ">";
}

uint64_t Timestamp::currentTimeMillis() {
	timeval tv;
	gettimeofday(&tv, NULL);

	// (second-value in milliseconds) + (microsecond value in milliseconds)
	return (((uint64_t) tv.tv_sec) * 1000l) + (tv.tv_usec / 1000l);
}

bool Timestamp::operator==(const Timestamp &other) const {
	return (inserted == other.inserted) && (updated == other.updated);
}

uint64_t Timestamp::getInserted() const {
	return inserted;
}

void Timestamp::setInserted(uint64_t inserted) {
	this->inserted = inserted;
}

uint64_t Timestamp::getUpdated() const {
	return updated;
}

void Timestamp::setUpdated(uint64_t updated) {
	this->updated = updated;
}

ostream& airobots::operator<<(ostream &o, Timestamp const &timestamp) {
	o << "Timestamp(inserted = " << timestamp.getInserted() << ", updated = "
			<< timestamp.getUpdated() << ")";
	return o;
}

void xmltio::convert(const airobots::Timestamp& timestamp, Location& loc) {

	stringstream insertedStream;
	stringstream updatedStream;
	insertedStream << timestamp.getInserted();
	updatedStream << timestamp.getUpdated();

	loc[XPath("INSERTED/@value")] = insertedStream.str();
	loc[XPath("UPDATED/@value")] = updatedStream.str();

}

template<>
airobots::Timestamp xmltio::extract<airobots::Timestamp>::get() const {

	stringstream insertedStream;
	stringstream updatedStream;

	string inserted = extract<string> (loc_[XPath("INSERTED/@value")]);
	insertedStream << inserted;
	uint64_t insertedTime;
	insertedStream >> insertedTime;
	string updated = extract<string> (loc_[XPath("UPDATED/@value")]);
	updatedStream << updated;
	uint64_t updatedTime;
	updatedStream >> updatedTime;

	return Timestamp(insertedTime, updatedTime);

}
