#ifndef UNITCONVERTER_H_
#define UNITCONVERTER_H_

#include <exception>
#include <string>

#include "LengthUnit.h"
#include "AngleUnit.h"
#include "TimeUnit.h"

#define PI 3.141592653589793

namespace airobots {

/**
 * This helper class converts any value from any unit to any other unit.
 * For more details, refer to the documentation of the Java implementation.
 */
class UnitConverter {
public:
	/**
	 * This exception indicates, that a conversion can not be done, because one of the given units
	 * is not known to the converter.
	 */
	class UnknownUnitException : public std::exception {
	private:
		std::string message;
	public:
		UnknownUnitException(std::string message = "UnknownUnitException") {
			this->message = message;
		}

		~UnknownUnitException() throw() {}

		virtual const char* what() const throw() {
			return message.c_str();
		}
	};

	static int convert(int, LengthUnit, LengthUnit);
	static long convert(long, LengthUnit, LengthUnit);
	static double convert(double, LengthUnit, LengthUnit);
	static float convert(float, LengthUnit, LengthUnit);

	static double convert(double, AngleUnit, AngleUnit);
	static float convert(float, AngleUnit, AngleUnit);

	static long convert(long, TimeUnit, TimeUnit);

private:

	static double convertFrom(double, LengthUnit);
	static double convertTo(double, LengthUnit);
	static float convertFrom(float, LengthUnit);
	static float convertTo(float, LengthUnit);

	static double convertFrom(double, AngleUnit);
	static double convertTo(double, AngleUnit);
	static float convertFrom(float, AngleUnit);
	static float convertTo(float, AngleUnit);

	static long convertFrom(long, TimeUnit);
	static long convertTo(long, TimeUnit);

	static std::string unitName(LengthUnit);
	static std::string unitName(AngleUnit);
	static std::string unitName(TimeUnit);
};

}

#endif /* UNITCONVERTER_H_ */
