/*
 * Voice.cpp
 *
 *  Created on: Aug 21, 2009
 *      Author: jwienke
 */

#include "Voice.h"

using namespace std;
using namespace airobots;
using namespace xmltio;

Voice::Voice() :
	angle(0), runtimeDifference(0), angleVariance(0),
			runtimeDifferenceVariance(0) {
}

Voice::~Voice() {
}

std::string Voice::getBaseTag() {
	return "VOICE";
}

std::string Voice::getTemplate() {
	return string("<") + getBaseTag() + ">" + "<GENERATOR/>"
			+ Timestamp::getTemplate()
			+ "<POSITION angle=\"\" angleVariance=\"\" runtimeDifference=\"\" runtimeDifferenceVariance=\"\"/>"
				"</" + getBaseTag() + ">";
}

std::string Voice::getPerceptName() {
	return "voice";
}

float Voice::getAngle() const {
	return angle;
}

void Voice::setAngle(float angle) {
	this->angle = angle;
}

float Voice::getAngleVariance() const {
	return angleVariance;
}

void Voice::setAngleVariance(float angleVariance) {
	this->angleVariance = angleVariance;
}

float Voice::getRuntimeDifference() const {
	return runtimeDifference;
}

void Voice::setRuntimeDifference(float runtimeDifference) {
	this->runtimeDifference = runtimeDifference;
}

float Voice::getRuntimeDifferenceVariance() const {
	return runtimeDifferenceVariance;
}

void Voice::setRuntimeDifferenceVariance(float runtimeDifferenceVariance) {
	this->runtimeDifferenceVariance = runtimeDifferenceVariance;
}

bool Voice::operator==(const Voice &other) const {

	return Type::operator==(other) && (other.angle == angle)
			&& (other.runtimeDifference == runtimeDifference)
			&& (other.angleVariance == angleVariance)
			&& (other.runtimeDifferenceVariance == runtimeDifferenceVariance);

}

std::ostream& airobots::operator<<(std::ostream& o, Voice const& voice) {
	return o << "Voice [angle = " << voice.getAngle() << ", angleVariance = "
			<< voice.getAngleVariance() << ", runtimeDifference = "
			<< voice.getRuntimeDifference() << ", runtimeDifferenceVariance = "
			<< voice.getRuntimeDifferenceVariance() << "]";
}

void Voice::sanitizeLocation(xmltio::Location &loc) const {
	
}

void Voice::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	loc[XPath("POSITION/@angle")] = getAngle();
	loc[XPath("POSITION/@angleVariance")] = getAngleVariance();

	loc[XPath("POSITION/@runtimeDifference")] = getRuntimeDifference();
	loc[XPath("POSITION/@runtimeDifferenceVariance")]
			= getRuntimeDifferenceVariance();

}

void Voice::fromLocation(const xmltio::Location &loc_, Voice &voice) {

	Type::fromLocation(loc_, voice);

	voice.setAngle(extract<float> (loc_[XPath("POSITION/@angle")]));
	voice.setAngleVariance(extract<float> (
			loc_[XPath("POSITION/@angleVariance")]));

	voice.setRuntimeDifference(extract<float> (loc_[XPath(
			"POSITION/@runtimeDifference")]));
	voice.setRuntimeDifferenceVariance(extract<float> (loc_[XPath(
			"POSITION/@runtimeDifferenceVariance")]));

}

