/*
 * GlobalPlannerStateChangeTest.cpp
 *
 *  Created on: Jul 20, 2009
 *      Author: jwienke
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include "btl/GlobalPlannerStateChange.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double tolerance = 0.0001;

BOOST_AUTO_TEST_SUITE(GlobalPlannerStateChangeTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		GlobalPlannerStateChange original("test", GlobalPlannerStateChange::PATH_BLOCKED);

		Location originalLoc(GlobalPlannerStateChange::getTemplate(), GlobalPlannerStateChange::getBaseTag());
		originalLoc = original;

		cout << originalLoc.getDocumentText(true);

		GlobalPlannerStateChange parsed = extract<GlobalPlannerStateChange>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_EQUAL(parsed.getNewState(), original.getNewState());

	} catch (std::exception e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("GlobalPlannerStateChange.xml"), GlobalPlannerStateChange::getBaseTag());
	GlobalPlannerStateChange parsed = extract<GlobalPlannerStateChange>(fileContents);

	BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_EQUAL(GlobalPlannerStateChange::PLANNING, parsed.getNewState());

}

BOOST_AUTO_TEST_SUITE_END()
