/*
 * GoalReachedDataTest.cpp
 *
 *  Created on: Jul 20, 2009
 *      Author: jwienke
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include "btl/GoalReachedData.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double tolerance = 0.0001;

BOOST_AUTO_TEST_SUITE(GoalReachedDataTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		GoalReachedData original("test", true);
		original.setTimestamp(Timestamp(123, 456));

		Location originalLoc(GoalReachedData::getTemplate(), GoalReachedData::getBaseTag());
		originalLoc = original;

		cout << originalLoc.getDocumentText(true) << endl;

		GoalReachedData parsed = extract<GoalReachedData>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_EQUAL(parsed.isGoalReached(), original.isGoalReached());

	} catch (std::exception e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("GoalReached.xml"), GoalReachedData::getBaseTag());
	GoalReachedData parsed = extract<GoalReachedData>(fileContents);

	BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_EQUAL(true, parsed.isGoalReached());

}

BOOST_AUTO_TEST_SUITE_END()
