/*
 * KatanaGripperData.cpp
 *
 *  Created on: May 25, 2010
 *      Author: ttoenige
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>
#include <string>

#include <sys/time.h>
#include <time.h>

#include "btl/KatanaGripperData.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(KatanaGripperDataTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility)
{
	try
	{

		KatanaGripperData original;
		original.setGenerator("test");
		original.setTimestamp(Timestamp(535234234));
		original.setForceRightInsideNear(1);
		original.setForceRightInsideFar(2);
		original.setInfraredRightOutside(3);
		original.setInfraredRightFront(4);
		original.setInfraredRightInsideNear(5);
		original.setInfraredRightInsideFar(6);
		original.setForceLeftInsideNear(7);
		original.setForceLeftInsideFar(8);
		original.setInfraredLeftOutside(9);
		original.setInfraredLeftFront(10);
		original.setInfraredLeftInsideNear(11);
		original.setInfraredLeftInsideFar(12);
		original.setInfraredMiddle(13);

		cout << KatanaGripperData::getTemplate() << endl;
		cout << KatanaGripperData::getBaseTag() << endl;

		
		Location originalLoc(KatanaGripperData::getTemplate(), KatanaGripperData::getBaseTag());

		originalLoc = original;

		cout << originalLoc.getDocumentText(true) << endl;

		KatanaGripperData parsed = extract<KatanaGripperData>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_EQUAL(parsed.getForceRightInsideNear(), original.getForceRightInsideNear());
		BOOST_REQUIRE_EQUAL(parsed.getForceRightInsideFar(), original.getForceRightInsideFar());
		BOOST_REQUIRE_EQUAL(parsed.getInfraredRightOutside(), original.getInfraredRightOutside());
		BOOST_REQUIRE_EQUAL(parsed.getInfraredRightFront(), original.getInfraredRightFront() );
		BOOST_REQUIRE_EQUAL(parsed.getInfraredRightInsideNear(), original.getInfraredRightInsideNear());
		BOOST_REQUIRE_EQUAL(parsed.getInfraredRightInsideFar(), original.getInfraredRightInsideFar());
		BOOST_REQUIRE_EQUAL(parsed.getForceLeftInsideNear(), original.getForceLeftInsideNear());
		BOOST_REQUIRE_EQUAL(parsed.getForceLeftInsideFar(), original.getForceLeftInsideFar());
		BOOST_REQUIRE_EQUAL(parsed.getInfraredLeftOutside(), original.getInfraredLeftOutside());
		BOOST_REQUIRE_EQUAL(parsed.getInfraredLeftFront(), original.getInfraredLeftFront() );
		BOOST_REQUIRE_EQUAL(parsed.getInfraredLeftInsideNear(), original.getInfraredLeftInsideNear());
		BOOST_REQUIRE_EQUAL(parsed.getInfraredLeftInsideFar(), original.getInfraredLeftInsideFar());
		BOOST_REQUIRE_EQUAL(parsed.getInfraredMiddle(), original.getInfraredMiddle());

	}
	catch (std::exception e)
	{
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility)
{
	Location fileContents(readFile("KatanaGripperData.xml"), KatanaGripperData::getBaseTag());
	KatanaGripperData parsed = extract<KatanaGripperData>(fileContents);

	BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_EQUAL(1, parsed.getForceRightInsideNear());
	BOOST_REQUIRE_EQUAL(2, parsed.getForceRightInsideFar());
	BOOST_REQUIRE_EQUAL(3, parsed.getInfraredRightOutside());
	BOOST_REQUIRE_EQUAL(4, parsed.getInfraredRightFront());
	BOOST_REQUIRE_EQUAL(5, parsed.getInfraredRightInsideNear());
	BOOST_REQUIRE_EQUAL(6, parsed.getInfraredRightInsideFar());
	BOOST_REQUIRE_EQUAL(7, parsed.getForceLeftInsideNear());
	BOOST_REQUIRE_EQUAL(8, parsed.getForceLeftInsideFar());
	BOOST_REQUIRE_EQUAL(9, parsed.getInfraredLeftOutside());
	BOOST_REQUIRE_EQUAL(10, parsed.getInfraredLeftFront());
	BOOST_REQUIRE_EQUAL(11, parsed.getInfraredLeftInsideNear());
	BOOST_REQUIRE_EQUAL(12, parsed.getInfraredLeftInsideFar());
	BOOST_REQUIRE_EQUAL(13, parsed.getInfraredMiddle());

}

BOOST_AUTO_TEST_SUITE_END()
