/*
 * KatanaPoseDataTest.cpp
 *
 *  Created on: May 25, 2010
 *      Author: ttoenige
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include <sys/time.h>
#include <time.h>

#include "btl/KatanaPoseData.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(KatanaPoseDataTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility)
{
	try
	{

		KatanaPoseData original;
		original.setGenerator("test");
		original.setTimestamp(Timestamp(535234234));
		original.setX(222);
		original.setY(333);
		original.setZ(444);
		original.setPhi(1.11);
		original.setPsi(0.55);
		original.setTheta(1.23);

		Location originalLoc(KatanaPoseData::getTemplate(), KatanaPoseData::getBaseTag());
		originalLoc = original;

		cout << originalLoc.getDocumentText(true) << endl;

		KatanaPoseData parsed = extract<KatanaPoseData>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_EQUAL(parsed.getX(), original.getX());
		BOOST_REQUIRE_EQUAL(parsed.getY(), original.getY());
		BOOST_REQUIRE_EQUAL(parsed.getZ(), original.getZ());
		BOOST_REQUIRE_EQUAL(parsed.getTheta(), original.getTheta() );
		BOOST_REQUIRE_EQUAL(parsed.getPhi(), original.getPhi());
		BOOST_REQUIRE_EQUAL(parsed.getPsi(), original.getPsi());

	}
	catch (std::exception e)
	{
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility)
{
	Location fileContents(readFile("KatanaPoseData.xml"), KatanaPoseData::getBaseTag());
	KatanaPoseData parsed = extract<KatanaPoseData>(fileContents);

	BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_EQUAL(232, parsed.getX());
	BOOST_REQUIRE_EQUAL(343, parsed.getY());
	BOOST_REQUIRE_EQUAL(454, parsed.getZ());
	BOOST_REQUIRE_EQUAL(1.34, parsed.getPhi());
	BOOST_REQUIRE_EQUAL(1.23, parsed.getPsi());
	BOOST_REQUIRE_EQUAL(0.23, parsed.getTheta());
}

BOOST_AUTO_TEST_SUITE_END()
