#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include "btl/LaserData.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.01;

BOOST_AUTO_TEST_SUITE(LaserDataTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		LaserData original;
		original.setGenerator("test");
		original.setTimestamp(Timestamp(123, 456));
		timeval now;
		gettimeofday(&now, NULL);
		original.setReadTime(now);
		vector<float> points;
		for (unsigned int i = 0; i < 35; i++) {
			points.push_back(((float)rand() / RAND_MAX) * 30.0);
		}
		original.setLaserPoints(points);
		BOOST_REQUIRE_EQUAL(original.getNumLaserPoints(), points.size());

		Location originalLoc(LaserData::getTemplate(), LaserData::getBaseTag());
		originalLoc = original;

		cout << originalLoc.getDocumentText(true) << endl;

		LaserData parsed = extract<LaserData>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_EQUAL(parsed.getReadTime(), original.getReadTime());
		BOOST_REQUIRE_EQUAL(parsed.getReadTimeval().tv_sec, original.getReadTimeval().tv_sec);
		BOOST_REQUIRE_EQUAL(parsed.getReadTimeval().tv_usec, original.getReadTimeval().tv_usec);
		BOOST_REQUIRE_EQUAL(parsed.getReadTimeSeconds(), original.getReadTimeSeconds());
		BOOST_REQUIRE_EQUAL(parsed.getReadTimeMicroSeconds(), original.getReadTimeMicroSeconds());
		BOOST_REQUIRE_EQUAL(parsed.getNumLaserPoints(), original.getNumLaserPoints());
		for (unsigned int i = 0; i < parsed.getNumLaserPoints(); i++) {
			BOOST_REQUIRE_CLOSE(parsed.getLaserPoints()[i], original.getLaserPoints()[i], TOLERANCE);
		}

	} catch (std::exception e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("Laser.xml"), LaserData::getBaseTag());
	LaserData parsed = extract<LaserData>(fileContents);

	BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_EQUAL(MicroTimestamp(123213412, 335435), parsed.getReadTime());
	BOOST_REQUIRE_EQUAL(5, parsed.getNumLaserPoints());
	BOOST_REQUIRE_CLOSE((float) 0.01, parsed.getLaserPoints()[0], 0.00001);
	BOOST_REQUIRE_CLOSE((float) 0.002, parsed.getLaserPoints()[1], 0.00001);
	BOOST_REQUIRE_CLOSE((float) 0.003, parsed.getLaserPoints()[2], 0.00001);
	BOOST_REQUIRE_CLOSE((float) 0.004, parsed.getLaserPoints()[3], 0.00001);
	BOOST_REQUIRE_CLOSE((float) 0.005, parsed.getLaserPoints()[4], 0.00001);

}

BOOST_AUTO_TEST_SUITE_END()
