/*
 * MovementDataTest.cpp
 *
 *  Created on: Jul 20, 2009
 *      Author: jwienke
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <exception>
#include <stdexcept>
#include <iostream>

#include <sys/time.h>
#include <time.h>

#include "btl/MicroTimestamp.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(MicroTimestampTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		timeval now;
		gettimeofday(&now, NULL);
		MicroTimestamp original(now);

		Location originalLoc(MicroTimestamp::getTemplate(), MicroTimestamp::getBaseTag());
		originalLoc = original;

		cout << originalLoc.getDocumentText(true) << endl;

		MicroTimestamp parsed = extract<MicroTimestamp>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getSeconds(), original.getSeconds());
		BOOST_REQUIRE_EQUAL(parsed.getMicroSeconds(), original.getMicroSeconds());

	} catch (std::exception e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testTimevalFromDouble) {

	timeval now;
	gettimeofday(&now, NULL);

	double time = (double) now.tv_sec + (double) (now.tv_usec / 1000000.0);

	timeval converted = MicroTimestamp::timevalFromDouble(time);

	BOOST_REQUIRE_EQUAL(now.tv_sec, converted.tv_sec);
	BOOST_REQUIRE_CLOSE((double) now.tv_usec, (double) converted.tv_usec, 10.0);

}

BOOST_AUTO_TEST_CASE(testUSecException) {

	BOOST_REQUIRE_THROW(MicroTimestamp kaputt(23, 1000001), std::domain_error);

	timeval wrong;
	wrong.tv_sec = 23;
	wrong.tv_usec = 1000001;

	BOOST_REQUIRE_THROW(MicroTimestamp kaputt2(wrong), std::domain_error);

}

BOOST_AUTO_TEST_SUITE_END()
