/*
 * ObjectDataTest.cpp
 *
 *  Created on: Apr 28, 2010
 *      Author: lziegler
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include <sys/time.h>
#include <time.h>

#include "btl/ObjectData.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(ObjectDataTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		ObjectData original;
		original.setClassLabel(string("smacks"));

		ObjectData::Hypothesis* originalHyp = new ObjectData::Hypothesis();
		originalHyp->setGenerator("test");
		originalHyp->setTimestamp(Timestamp(253543797,253543712));
		originalHyp->setReliability(0.1);

		original.addHypothesis(originalHyp);

		Location originalLoc(ObjectData::getTemplate(), ObjectData::getBaseTag());
		originalLoc = original;

		cout << originalLoc.getDocumentText(true) << endl;

		ObjectData parsed = extract<ObjectData>(originalLoc);
		ObjectData::Hypothesis* parsedHyp = *(parsed.getHypotheses().begin());

		BOOST_REQUIRE_EQUAL(parsedHyp->getGenerator(), originalHyp->getGenerator());
		BOOST_REQUIRE_EQUAL(parsedHyp->getTimestamp(), originalHyp->getTimestamp());
		BOOST_REQUIRE_CLOSE(parsedHyp->getReliability(), originalHyp->getReliability(), TOLERANCE);
		BOOST_REQUIRE_EQUAL(parsed.getClassLabel(), original.getClassLabel());

	} catch (std::exception e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("ObjectData.xml"), ObjectData::getBaseTag());
	ObjectData parsed = extract<ObjectData>(fileContents);
	ObjectData::Hypothesis* parsedHyp = *(parsed.getHypotheses().begin());

	BOOST_REQUIRE_EQUAL("BoostDetector", parsedHyp->getGenerator());
	BOOST_REQUIRE_EQUAL(253543797, parsedHyp->getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(253543712, parsedHyp->getTimestamp().getUpdated());
	BOOST_REQUIRE_CLOSE(0.6, parsedHyp->getReliability(), 0.0001);
	BOOST_REQUIRE_EQUAL("smacks", parsed.getClassLabel());

}

BOOST_AUTO_TEST_SUITE_END()
