/*
 * ObjectLocationDataTest.cpp
 *
 *  Created on: Apr 28, 2010
 *      Author: lziegler
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include <sys/time.h>
#include <time.h>

#include "btl/ObjectLocationData.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(ObjectLocationDataTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		ObjectLocationData original;

		original.setClassLabel(string("smacks"));

		vector<pair<int, int> > poly;
		poly.push_back(pair<int,int>(0,0));
		poly.push_back(pair<int,int>(0,2));
		poly.push_back(pair<int,int>(2,0));

		ObjectLocationData::Hypothesis* originalHyp = new ObjectLocationData::Hypothesis();
		originalHyp->setGenerator("test");
		originalHyp->setTimestamp(Timestamp(253543797,253543712));
		originalHyp->setReliability(0.1);
		originalHyp->setPolygon(poly);
		original.addHypothesis(originalHyp);

		Location originalLoc(ObjectLocationData::getTemplate(), ObjectLocationData::getBaseTag());
		originalLoc = original;

		cout << originalLoc.getDocumentText(true) << endl;

		ObjectLocationData parsed = extract<ObjectLocationData>(originalLoc);
		ObjectLocationData::Hypothesis* parsedHyp = *(parsed.getLocationHypotheses().begin());

		BOOST_REQUIRE_EQUAL(parsedHyp->getGenerator(), originalHyp->getGenerator());
		BOOST_REQUIRE_EQUAL(parsedHyp->getTimestamp(), originalHyp->getTimestamp());
		BOOST_REQUIRE_CLOSE(parsedHyp->getReliability(), originalHyp->getReliability(), TOLERANCE);
		BOOST_REQUIRE_EQUAL(parsed.getClassLabel(), original.getClassLabel());
		BOOST_REQUIRE_EQUAL(parsedHyp->getSensorType(), originalHyp->getSensorType());
		BOOST_REQUIRE_EQUAL(originalHyp->getPolygon().size(), parsedHyp->getPolygon().size());
		BOOST_REQUIRE_EQUAL(originalHyp->getPolygon()[0].first, parsedHyp->getPolygon()[0].first);
		BOOST_REQUIRE_EQUAL(originalHyp->getPolygon()[1].first, parsedHyp->getPolygon()[1].first);
		BOOST_REQUIRE_EQUAL(originalHyp->getPolygon()[2].first, parsedHyp->getPolygon()[2].first);

		BOOST_REQUIRE(original == parsed);
		parsed.setClassLabel("bla");
		BOOST_REQUIRE(!(original == parsed));

	} catch (std::exception e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {


	Location fileContents(readFile("ObjectLocationData.xml"), ObjectLocationData::getBaseTag());
	ObjectLocationData parsed = extract<ObjectLocationData>(fileContents);
	ObjectLocationData::Hypothesis* parsedHyp = *(parsed.getLocationHypotheses().begin());

	BOOST_REQUIRE_EQUAL("BoostDetector", parsedHyp->getGenerator());
	BOOST_REQUIRE_EQUAL(253543797, parsedHyp->getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(253543712, parsedHyp->getTimestamp().getUpdated());
	BOOST_REQUIRE_CLOSE(0.6, parsedHyp->getReliability(), 0.0001);
	BOOST_REQUIRE_EQUAL("smacks", parsed.getClassLabel());
	BOOST_REQUIRE_EQUAL("SONY_EVI", parsedHyp->getSensorType());
	BOOST_REQUIRE_EQUAL(3, parsedHyp->getPolygon().size());
	BOOST_REQUIRE_EQUAL(0, parsedHyp->getPolygon()[0].first);
	BOOST_REQUIRE_EQUAL(1, parsedHyp->getPolygon()[1].first);
	BOOST_REQUIRE_EQUAL(2, parsedHyp->getPolygon()[2].first);

}

BOOST_AUTO_TEST_SUITE_END()
