#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include "btl/OdometryData.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const float TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(OdometryDataTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		OdometryData original("test", 123.45, -234.45, 0.23);
		original.setTimestamp(Timestamp(123, 456));
		timeval now;
		gettimeofday(&now, NULL);
		original.setReadTime(now);

		Location originalLoc(OdometryData::getTemplate(), OdometryData::getBaseTag());
		originalLoc = original;

		OdometryData parsed = extract<OdometryData>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_CLOSE(parsed.getX(), original.getX(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getY(), original.getY(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getTheta(), original.getTheta(), TOLERANCE);
		BOOST_REQUIRE_EQUAL(parsed.getReadTime(), original.getReadTime());

	} catch (std::exception e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("RobotPosition.xml"), OdometryData::getBaseTag());
	OdometryData parsed = extract<OdometryData>(fileContents);

	BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_EQUAL(MicroTimestamp(123213412, 335435), parsed.getReadTime());
	BOOST_REQUIRE_CLOSE((float) 234.5, parsed.getX(), 0.00001);
	BOOST_REQUIRE_CLOSE((float) -6345.42, parsed.getY(), 0.00001);
	BOOST_REQUIRE_CLOSE((float) 0.34, parsed.getTheta(), 0.00001);

}

BOOST_AUTO_TEST_SUITE_END()
