#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include "btl/PositionData.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(PositionDataTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		PositionData original(123.45f, -234.45f, 0.23f);
		original.setGenerator("test");
		original.setTimestamp(Timestamp(123, 456));

		Location originalLoc(PositionData::getTemplate(), PositionData::getBaseTag());
		originalLoc = original;

		PositionData parsed = extract<PositionData>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_CLOSE(parsed.getX(), original.getX(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getY(), original.getY(), TOLERANCE);
		BOOST_REQUIRE_CLOSE(parsed.getTheta(), original.getTheta(), TOLERANCE);

	} catch (std::exception e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_SUITE_END()
