#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include "btl/Shirt.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double tolerance = 0.0001;

BOOST_AUTO_TEST_SUITE(ShirtTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		Shirt original;
		original.setGenerator("test");
		original.setTimestamp(Timestamp(123, 456));
		original.setAngle(21312.78);
		original.setDistance(-1.023);
		original.setAngleVariance(0.45);
		original.setDistanceVariance(1.34);

		Location originalLoc(Shirt::getTemplate(), Shirt::getBaseTag());
		originalLoc = original;

		Shirt parsed = extract<Shirt>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_CLOSE(parsed.getAngle(), original.getAngle(), tolerance);
		BOOST_REQUIRE_CLOSE(parsed.getDistance(), original.getDistance(), tolerance);
		BOOST_REQUIRE_CLOSE(parsed.getAngleVariance(), original.getAngleVariance(), tolerance);
		BOOST_REQUIRE_CLOSE(parsed.getDistanceVariance(), original.getDistanceVariance(), tolerance);

	} catch (std::exception & e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("Shirt.xml"), Shirt::getBaseTag());
	Shirt parsed = extract<Shirt>(fileContents);

	BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_CLOSE((float) 0.45, parsed.getAngle(), tolerance);
	BOOST_REQUIRE_CLOSE((float) 77.23, parsed.getDistance(), tolerance);
	BOOST_REQUIRE_CLOSE((float) 0.345, parsed.getAngleVariance(), tolerance);
	BOOST_REQUIRE_CLOSE((float) 1.34, parsed.getDistanceVariance(), tolerance);

}

BOOST_AUTO_TEST_SUITE_END()
