#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include "btl/UnitConverter.h"

using namespace airobots;
using namespace std;

BOOST_AUTO_TEST_SUITE(BinarySlamMapTest)

BOOST_AUTO_TEST_CASE(testConvert) {

	const float DELTA = 0.001f;

	// ############# LENGTH ###############

	// INTEGER
	BOOST_REQUIRE_EQUAL(1, UnitConverter::convert(1, MILLIMETER, MILLIMETER));
	BOOST_REQUIRE_EQUAL(0, UnitConverter::convert(1, MILLIMETER, CENTIMETER));
	BOOST_REQUIRE_EQUAL(0, UnitConverter::convert(1, MILLIMETER, METER));

	BOOST_REQUIRE_EQUAL(1000, UnitConverter::convert(1000, MILLIMETER, MILLIMETER));
	BOOST_REQUIRE_EQUAL(100, UnitConverter::convert(1000, MILLIMETER, CENTIMETER));
	BOOST_REQUIRE_EQUAL(1, UnitConverter::convert(1000, MILLIMETER, METER));

	BOOST_REQUIRE_EQUAL(10000, UnitConverter::convert(1000, CENTIMETER, MILLIMETER));
	BOOST_REQUIRE_EQUAL(1000, UnitConverter::convert(1000, CENTIMETER, CENTIMETER));
	BOOST_REQUIRE_EQUAL(10, UnitConverter::convert(1000, CENTIMETER, METER));

	BOOST_REQUIRE_EQUAL(1000000, UnitConverter::convert(1000, METER, MILLIMETER));
	BOOST_REQUIRE_EQUAL(100000, UnitConverter::convert(1000, METER, CENTIMETER));
	BOOST_REQUIRE_EQUAL(1000, UnitConverter::convert(1000, METER, METER));

	// LONG
	BOOST_REQUIRE_EQUAL(1l, UnitConverter::convert(1l, MILLIMETER, MILLIMETER));
	BOOST_REQUIRE_EQUAL(0l, UnitConverter::convert(1l, MILLIMETER, CENTIMETER));
	BOOST_REQUIRE_EQUAL(0l, UnitConverter::convert(1l, MILLIMETER, METER));

	BOOST_REQUIRE_EQUAL(1000l, UnitConverter::convert(1000l, MILLIMETER, MILLIMETER));
	BOOST_REQUIRE_EQUAL(100l, UnitConverter::convert(1000l, MILLIMETER, CENTIMETER));
	BOOST_REQUIRE_EQUAL(1l, UnitConverter::convert(1000l, MILLIMETER, METER));

	BOOST_REQUIRE_EQUAL(10000l, UnitConverter::convert(1000l, CENTIMETER, MILLIMETER));
	BOOST_REQUIRE_EQUAL(1000l, UnitConverter::convert(1000l, CENTIMETER, CENTIMETER));
	BOOST_REQUIRE_EQUAL(10l, UnitConverter::convert(1000l, CENTIMETER, METER));

	BOOST_REQUIRE_EQUAL(1000000l, UnitConverter::convert(1000l, METER, MILLIMETER));
	BOOST_REQUIRE_EQUAL(100000l, UnitConverter::convert(1000l, METER, CENTIMETER));
	BOOST_REQUIRE_EQUAL(1000l, UnitConverter::convert(1000l, METER, METER));

	// FLOAT
	BOOST_REQUIRE_CLOSE(1.0f, UnitConverter::convert(1.0f, MILLIMETER, MILLIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(0.1f, UnitConverter::convert(1.0f, MILLIMETER, CENTIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(0.001f, UnitConverter::convert(1.0f, MILLIMETER, METER), DELTA);

	BOOST_REQUIRE_CLOSE(1000.0f, UnitConverter::convert(1000.0f, MILLIMETER, MILLIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(100.0f, UnitConverter::convert(1000.0f, MILLIMETER, CENTIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(1.0f, UnitConverter::convert(1000.0f, MILLIMETER, METER), DELTA);

	BOOST_REQUIRE_CLOSE(10000.0f, UnitConverter::convert(1000.0f, CENTIMETER, MILLIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(1000.0f, UnitConverter::convert(1000.0f, CENTIMETER, CENTIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(10.0f, UnitConverter::convert(1000.0f, CENTIMETER, METER), DELTA);

	BOOST_REQUIRE_CLOSE(1000000.0f, UnitConverter::convert(1000.0f, METER, MILLIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(100000.0f, UnitConverter::convert(1000.0f, METER, CENTIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(1000.0f, UnitConverter::convert(1000.0f, METER, METER), DELTA);

	// DOUBLE
	BOOST_REQUIRE_CLOSE(1.0, UnitConverter::convert(1.0, MILLIMETER, MILLIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(0.1, UnitConverter::convert(1.0, MILLIMETER, CENTIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(0.001, UnitConverter::convert(1.0, MILLIMETER, METER), DELTA);

	BOOST_REQUIRE_CLOSE(1000.0, UnitConverter::convert(1000.0, MILLIMETER, MILLIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(100.0, UnitConverter::convert(1000.0, MILLIMETER, CENTIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(1.0, UnitConverter::convert(1000.0, MILLIMETER, METER), DELTA);

	BOOST_REQUIRE_CLOSE(10000.0, UnitConverter::convert(1000.0, CENTIMETER, MILLIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(1000.0, UnitConverter::convert(1000.0, CENTIMETER, CENTIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(10.0, UnitConverter::convert(1000.0, CENTIMETER, METER), DELTA);

	BOOST_REQUIRE_CLOSE(1000000.0, UnitConverter::convert(1000.0, METER, MILLIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(100000.0, UnitConverter::convert(1000.0, METER, CENTIMETER), DELTA);
	BOOST_REQUIRE_CLOSE(1000.0, UnitConverter::convert(1000.0, METER, METER), DELTA);


	// ############# ANGLE ###############

	// FLOAT
	BOOST_REQUIRE_CLOSE((float) PI, UnitConverter::convert(180.0f, DEGREE, RADIANT), DELTA);
	BOOST_REQUIRE_CLOSE((float) PI/2.0f, UnitConverter::convert(90.0f, DEGREE, RADIANT), DELTA);
	BOOST_REQUIRE_CLOSE((float) -PI/2.0f, UnitConverter::convert(-90.0f, DEGREE, RADIANT), DELTA);
	BOOST_REQUIRE_CLOSE(0.0f, UnitConverter::convert(0.0f, DEGREE, RADIANT), DELTA);

	BOOST_REQUIRE_CLOSE(180.0f, UnitConverter::convert((float) PI, RADIANT, DEGREE), DELTA);
	BOOST_REQUIRE_CLOSE(90.0f, UnitConverter::convert((float) PI/2.0f, RADIANT, DEGREE), DELTA);
	BOOST_REQUIRE_CLOSE(-90.0f, UnitConverter::convert((float) -PI/2.0f, RADIANT, DEGREE), DELTA);
	BOOST_REQUIRE_CLOSE(0.0f, UnitConverter::convert(0.0f, RADIANT, DEGREE), DELTA);

	BOOST_REQUIRE_CLOSE(0.0f, UnitConverter::convert(0.0f, RADIANT, RADIANT), DELTA);
	BOOST_REQUIRE_CLOSE(1.0f, UnitConverter::convert(1.0f, RADIANT, RADIANT), DELTA);
	BOOST_REQUIRE_CLOSE(0.0f, UnitConverter::convert(0.0f, DEGREE, DEGREE), DELTA);
	BOOST_REQUIRE_CLOSE(1.0f, UnitConverter::convert(1.0f, DEGREE, DEGREE), DELTA);

	// DOUBLE
	BOOST_REQUIRE_CLOSE(PI, UnitConverter::convert(180.0, DEGREE, RADIANT), DELTA);
	BOOST_REQUIRE_CLOSE(PI/2.0, UnitConverter::convert(90.0, DEGREE, RADIANT), DELTA);
	BOOST_REQUIRE_CLOSE(-PI/2.0, UnitConverter::convert(-90.0, DEGREE, RADIANT), DELTA);
	BOOST_REQUIRE_CLOSE(0.0, UnitConverter::convert(0.0, DEGREE, RADIANT), DELTA);

	BOOST_REQUIRE_CLOSE(180.0, UnitConverter::convert(PI, RADIANT, DEGREE), DELTA);
	BOOST_REQUIRE_CLOSE(90.0, UnitConverter::convert(PI/2.0, RADIANT, DEGREE), DELTA);
	BOOST_REQUIRE_CLOSE(-90.0, UnitConverter::convert(-PI/2.0, RADIANT, DEGREE), DELTA);
	BOOST_REQUIRE_CLOSE(0.0, UnitConverter::convert(0.0, RADIANT, DEGREE), DELTA);

	BOOST_REQUIRE_CLOSE(0.0, UnitConverter::convert(0.0, RADIANT, RADIANT), DELTA);
	BOOST_REQUIRE_CLOSE(1.0, UnitConverter::convert(1.0, RADIANT, RADIANT), DELTA);
	BOOST_REQUIRE_CLOSE(0.0, UnitConverter::convert(0.0, DEGREE, DEGREE), DELTA);
	BOOST_REQUIRE_CLOSE(1.0, UnitConverter::convert(1.0, DEGREE, DEGREE), DELTA);

	// ############# TIME ###############

	BOOST_REQUIRE_EQUAL(1l, UnitConverter::convert(1l, MICROSECONDS, MICROSECONDS));
	BOOST_REQUIRE_EQUAL(1000l, UnitConverter::convert(1l, MILLISECONDS, MICROSECONDS));
	/*
	BOOST_REQUIRE_EQUAL(1000l * 1000l, UnitConverter::convert(1l, SECONDS, MICROSECONDS));
	BOOST_REQUIRE_EQUAL(1000l * 1000l * 60l, UnitConverter::convert(1l, MINUTES, MICROSECONDS));
	BOOST_REQUIRE_EQUAL(1000l * 1000l * 60l * 60l, UnitConverter::convert(1l, HOURS, MICROSECONDS));
	BOOST_REQUIRE_EQUAL(1000l * 1000l * 60l * 60l * 24l, UnitConverter::convert(1l, DAYS, MICROSECONDS));

	BOOST_REQUIRE_EQUAL(0l, UnitConverter::convert(1l, MICROSECONDS, DAYS));
	BOOST_REQUIRE_EQUAL(0l, UnitConverter::convert(1l, MILLISECONDS, DAYS));
	BOOST_REQUIRE_EQUAL(0l, UnitConverter::convert(1l, SECONDS, DAYS));
	BOOST_REQUIRE_EQUAL(0l, UnitConverter::convert(1l, MINUTES, DAYS));
	BOOST_REQUIRE_EQUAL(0l, UnitConverter::convert(1l, HOURS, DAYS));
	BOOST_REQUIRE_EQUAL(1l, UnitConverter::convert(1l, DAYS, DAYS));

	BOOST_REQUIRE_EQUAL(1l, UnitConverter::convert(1000l * 1000L, MICROSECONDS, SECONDS));
	BOOST_REQUIRE_EQUAL(1l, UnitConverter::convert(1000l, MILLISECONDS, SECONDS));
	BOOST_REQUIRE_EQUAL(1l, UnitConverter::convert(1l, SECONDS, SECONDS));
	BOOST_REQUIRE_EQUAL(60l, UnitConverter::convert(1l, MINUTES, SECONDS));
	BOOST_REQUIRE_EQUAL(60l * 60l, UnitConverter::convert(1l, HOURS, SECONDS));
	BOOST_REQUIRE_EQUAL(60l * 60l * 24l, UnitConverter::convert(1l, DAYS, SECONDS));
*/
}

BOOST_AUTO_TEST_SUITE_END()
