#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include "btl/Voice.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double tolerance = 0.0001;

BOOST_AUTO_TEST_SUITE(VoiceTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		Voice original;
		original.setGenerator("test");
		original.setTimestamp(Timestamp(123, 456));
		original.setAngle(21312.78);
		original.setRuntimeDifference(-1.023);
		original.setAngleVariance(0.45);
		original.setRuntimeDifferenceVariance(1.34);

		Location originalLoc(Voice::getTemplate(), Voice::getBaseTag());
		originalLoc = original;

		cout << originalLoc.getDocumentText(true);

		Voice parsed = extract<Voice>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_CLOSE(parsed.getAngle(), original.getAngle(), tolerance);
		BOOST_REQUIRE_CLOSE(parsed.getRuntimeDifference(), original.getRuntimeDifference(), tolerance);
		BOOST_REQUIRE_CLOSE(parsed.getAngleVariance(), original.getAngleVariance(), tolerance);
		BOOST_REQUIRE_CLOSE(parsed.getRuntimeDifferenceVariance(), original.getRuntimeDifferenceVariance(), tolerance);

		BOOST_REQUIRE_EQUAL(parsed, original);
		parsed.setGenerator("trash");
		BOOST_REQUIRE(!(parsed == original));
		parsed.setGenerator(original.getGenerator());
		parsed.setRuntimeDifference(999.9);
		BOOST_REQUIRE(!(parsed == original));

	} catch (std::exception & e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("Voice.xml"), Voice::getBaseTag());
	Voice parsed = extract<Voice>(fileContents);

	BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_CLOSE((float) 0.45, parsed.getAngle(), tolerance);
	BOOST_REQUIRE_CLOSE((float) 77.23, parsed.getRuntimeDifference(), tolerance);
	BOOST_REQUIRE_CLOSE((float) 0.345, parsed.getAngleVariance(), tolerance);
	BOOST_REQUIRE_CLOSE((float) 1.34, parsed.getRuntimeDifferenceVariance(), tolerance);

}

BOOST_AUTO_TEST_SUITE_END()
