/**
 * A simple example program demonstrating how to preserve the dbxml id when not
 * reusing the location.
 */

#include <iostream>

#include <Memory/Interface.hpp>

#include "btl/Legs.h"

using namespace airobots;
using namespace std;
using namespace xmltio;
using namespace memory::interface;

int main(int argc, char **argv) {

	if (argc != 2) {
		cerr << "usage: " << argv[0] << " [memory name]" << endl;
		return 1;
	}

	// create a data class
	Legs original;
	original.setGenerator("test");
	original.setTimestamp(Timestamp(123, 456));
	original.setAngle(21312.78);
	original.setDistance(-1.023);
	original.setAngleVariance(0.45);
	original.setDistanceVariance(1.34);
	original.setNumInvolved(2134);
	original.setInvolved1(4543);
	original.setInvolved2(1223);
	original.setSelected(true);

	Location originalLoc(Legs::getTemplate(), Legs::getBaseTag());
	originalLoc = original;

	// display its contents
	cout << "inserting:" << endl << originalLoc.getDocumentText(true);

	// add it to the memory
	stringstream memoryName;
	memoryName << "xcf:" << argv[1];
	MemoryInterface::pointer memory = MemoryInterface::getInstance(memoryName.str());
	memory->insert(originalLoc.getDocumentText());

	// now query it back
	ResultsPtr queryResults = memory->query("/");
	if (!queryResults->hasNext()) {
		cerr << "something really weird happened with the memory" << endl;
		return 1;
	}
	string resultString;
	queryResults->next(resultString);
	Location resultLocation(resultString, Legs::getBaseTag());
	Legs parsedLegs = extract<Legs>(resultLocation);

	cout << "queried:" << endl << resultLocation.getDocumentText(true);

	// now change the parsed results and preserve the id for a replace
	parsedLegs.setDistance(23456.435);
	Location replaceLocation(Legs::getTemplate(), Legs::getBaseTag());
	replaceLocation = parsedLegs;
	replaceLocation.add(resultLocation[XPath("@dbxml:id")]);

	cout << "will be replaced with:" << endl << replaceLocation.getDocumentText(true);

	// replace the memory contents
	memory->replace(replaceLocation.getDocumentText());

	sleep(3);

}

