package de.unibi.airobots.btl.data;

import de.unibi.airobots.btl.common.ElementParser;
import de.unibi.airobots.btl.common.Timestamp;

import java.util.List;
import java.util.ArrayList;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;

// ==========================================================================
// We should adapt the output of the boost detectors to the btl library. By
// now the classes for the found faces are adapted to the original output of
// the boost detector to make sure other components could continue work.
// ==========================================================================

/** 
 * A list of found faces. This class is based on the 
 * original output of the boost detector.
 * 
 * @author: dschulze <dschulze@techfak.uni-bielefeld.de>
 */
public class FaceList extends Type {

	/** A list of all faces in this face list. */
	private List<Face> faces;

	/** Constructor. */
	public FaceList() {
		faces = new ArrayList<Face>();
	}

	public void addFace(Face face) {
		faces.add(face);
	}

	public List<Face> getFaces() {
		return faces;
	}

	public static String getBaseTag() {
		return "OBJECTS";
	}

	/** @{inheritDoc} */
	@Override
	public void fillInto(Element parent) {

		parent.addAttribute(new Attribute("creator", getGenerator()));

		for (Face face : faces) {

			Element faceElement = new Element(Face.getBaseTag());
			face.fillInto(faceElement);

			parent.appendChild(faceElement);

		}

	}

	/** @{inheritDoc} */
	public static void fromElement(Element element, FaceList list) {

		Nodes nodes = element.query(Face.getBaseTag());
		int nrNodes = nodes.size();
		for (int i = 0; i < nrNodes; i++) {

			Node node = nodes.get(i);
			if (node instanceof Element) {

				Element elem = (Element) node;

				Face face = new Face();
				Face.fromElement(elem, face);

				list.addFace(face);

			}

		}

	}	

}
