package de.unibi.airobots.btl.data;

import de.unibi.airobots.btl.common.ElementParser;
import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;

/** 
 * The legs class stores information about leg hypothesis
 * coming from the leg detector.
 * 
 * @author: dschulze <dschulze@techfak.uni-bielefeld.de>
 */
public class Legs extends Type {

	private double angle;
	private double distance;
	private double angleVariance;
	private double distanceVariance;
	private int numInvolved;
	private int[] involved;
	private boolean pair;

	public Legs() {

		involved = new int[2];
		involved[0] = -1;
		involved[1] = -1;

	}

	public void setAngle(double angle) {
		this.angle = angle;
	}
	
	public double getAngle() {
		return angle;
	}

	public void setAngleVariance(double angleVariance) {
		this.angleVariance = angleVariance;
	}
		
	public double getAngleVariance() {
		return angleVariance;
	}	

	public void setDistance(double distance) {
		this.distance = distance;
	}

	public double getDistance() {
		return distance;
	}

	public void setDistanceVariance(double distanceVariance) {
		this.distanceVariance = distanceVariance;
	}

	public double getDistanceVariance() {
		return distanceVariance;
	}

	public void setNumInvolved(int numInvolved) {
		this.numInvolved = numInvolved;
	}

	public int getNumInvolved() {
		return numInvolved;
	}

	public void setInvolved1(int involved) {
		this.involved[0] = involved;
	}

	public int getInvolved1() {
		return involved[0];
	}

	public void setInvolved2(int involved) {
		this.involved[1] = involved;
	}

	public int getInvolved2() {
		return involved[1];
	}

	public void setPair(boolean pair) {
		this.pair = pair;
	}

	public boolean isPair() {
		return pair;
	}
	
	public static String getBaseTag() {
		return "LEGS";
	}

	/** @{inheritDoc} */
	public static void fromElement(Element element, Legs legs) {

		Type.fromElement(element, legs);

		Nodes nodes = element.query("*");
		int nrNodes = nodes.size();
		for (int i = 0; i < nrNodes; i++) {
			Node node = nodes.get(i);
			if (node instanceof Element) {

				Element elem = (Element) node;
				if (elem.getLocalName().equals("POSITION")) {	

					double tmp = ElementParser.getDoubleAttributeValue(elem, "angle");
					legs.setAngle(tmp);
					
					tmp = ElementParser.getDoubleAttributeValue(elem, "angleVariance");
					legs.setAngleVariance(tmp);

					tmp = ElementParser.getDoubleAttributeValue(elem, "distance");
					legs.setDistance(tmp);
			
					tmp = ElementParser.getDoubleAttributeValue(elem, "distanceVariance");
					legs.setDistanceVariance(tmp);	

				}

				if (elem.getLocalName().equals("SHARING")) {

					int tmp = ElementParser.getIntAttributeValue(elem, "nrInvolved");
					legs.setNumInvolved(tmp);
				
					tmp = ElementParser.getIntAttributeValue(elem, "involved1");
					legs.setInvolved1(tmp);
		
					tmp = ElementParser.getIntAttributeValue(elem, "involved2");
					legs.setInvolved2(tmp);

					boolean pair = ElementParser.getBooleanAttributeValue(elem, "isPair");
					legs.setPair(pair);				

				}

			}	

		}

	}

	/** @{inheritDoc} */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		Element positionElement = new Element("POSITION");
		positionElement.addAttribute(new Attribute("angle", String.valueOf(getAngle())));
		positionElement.addAttribute(new Attribute("angleVariance", String.valueOf(getAngleVariance())));
		positionElement.addAttribute(new Attribute("distance", String.valueOf(getDistance())));
		positionElement.addAttribute(new Attribute("distanceVariance", String.valueOf(getDistanceVariance())));
		
		parent.appendChild(positionElement);

		Element sharingElement = new Element("SHARING");
		sharingElement.addAttribute(new Attribute("nrInvolved", String.valueOf(getNumInvolved())));
		sharingElement.addAttribute(new Attribute("involved1", String.valueOf(getInvolved1())));
		sharingElement.addAttribute(new Attribute("involved2", String.valueOf(getInvolved2())));
		sharingElement.addAttribute(new Attribute("isPair", String.valueOf(isPair())));

		parent.appendChild(sharingElement);

	}

}
