package de.unibi.airobots.btl.data;

import java.util.ArrayList;

import de.unibi.airobots.btl.common.ElementParser;

import nu.xom.Nodes;
import nu.xom.Node;
import nu.xom.Attribute;
import nu.xom.Element;

/**
 * A collection of person data.
 * 
 * @author: Denis Schulze <dschulze@techfak.uni-bielefeld.de>
 */
public class PersonDataList extends Type {

	private ArrayList<PersonData> personData;

	/** Constructor. */
	public PersonDataList() {
		super();
		personData = new ArrayList<PersonData>();
	}

	/** Adds new person data to this list. */
	public void addPersonData(PersonData data) {
		personData.add(data);
	}

	/** Returns the person data from the list. */
	public PersonData getPersonData(int index) {
		if (index > personData.size())
			return null;

		return personData.get(index);
	}
	
	/** Removes all person data from this list. */
	public void clearList() {
		personData.clear();
	}

	/** Returns the number of stored person data. */
	public int size() {
		return personData.size();
	}

	/** Returns the base tag as a string. */
	public static String getBaseTag() {
		return "PERSONS";
	}

	/** {@inheritDoc} */
	public static void fromElement(Element element, PersonDataList type) {

		Type.fromElement(element, type);

		Nodes nodes = element.query(PersonData.getBaseTag());
		int nrNodes = nodes.size();
		for (int i = 0; i < nrNodes; i++) {

			Node node = nodes.get(i);
			if (node instanceof Element) {

				Element personElement = (Element) node;

				PersonData parsed = new PersonData();
				PersonData.fromElement(personElement, parsed);
				type.addPersonData(parsed);				
	
			}

		}

	}

	/** {@inheritDoc} */
	@Override
	public void fillInto(Element parent) {
		super.fillInto(parent);

		int nrPersonData = personData.size();
		for (int i = 0; i < nrPersonData; i++) {

			Element personDataElement = new Element(PersonData.getBaseTag());
			personData.get(i).fillInto(personDataElement);
			parent.appendChild(personDataElement);

		}	

	}


}
