package de.unibi.airobots.btl.data;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Elements;
import nu.xom.ParsingException;
import de.unibi.airobots.btl.common.ElementParser;
import de.unibi.airobots.btl.common.PrecisePolygon;
import de.unibi.airobots.btl.units.AngleUnit;
import de.unibi.airobots.btl.units.LengthUnit;
import de.unibi.airobots.btl.units.UnitConverter;

public class PlaneData extends RegionData {

	public static final String TRANSLATION_TAG_NAME = "TRANSLATION";
	public static final String ROTATION_TAG_NAME = "NORMALROTATION";

	double translationX;
	double translationY;
	double translationZ;
	double normalrotaionX;
	double normalrotaionY;
	double normalrotaionZ;

	public PlaneData() {
		super();
	}

	public PlaneData(PrecisePolygon polygon) {
		super(polygon);
	}

	public PlaneData(PrecisePolygon polygon, Scope scope) {
		super(polygon, scope);
	}

	public PlaneData(Scope scope) {
		super(scope);
	}

	public void setTranslation(double translationX, double translationY, double translationZ,
			LengthUnit unit) {
		this.translationX = UnitConverter.convert(translationX, unit, LengthUnit.MILLIMETER);
		this.translationY = UnitConverter.convert(translationY, unit, LengthUnit.MILLIMETER);
		this.translationZ = UnitConverter.convert(translationZ, unit, LengthUnit.MILLIMETER);
	}

	public double getTranslationX(LengthUnit unit) {
		return UnitConverter.convert(translationX, LengthUnit.MILLIMETER, unit);
	}

	public double getTranslationY(LengthUnit unit) {
		return UnitConverter.convert(translationY, LengthUnit.MILLIMETER, unit);
	}

	public double getTranslationZ(LengthUnit unit) {
		return UnitConverter.convert(translationZ, LengthUnit.MILLIMETER, unit);
	}

	public void setNormalRotationX(double normalrotaionX, double normalrotaionY,
			double normalrotaionZ, AngleUnit unit) {
		this.normalrotaionX = UnitConverter.convert(normalrotaionX, unit, AngleUnit.RADIANT);
		this.normalrotaionY = UnitConverter.convert(normalrotaionY, unit, AngleUnit.RADIANT);
		this.normalrotaionZ = UnitConverter.convert(normalrotaionZ, unit, AngleUnit.RADIANT);
	}

	public double getNormalRotationX(AngleUnit unit) {
		return UnitConverter.convert(normalrotaionX, AngleUnit.RADIANT, unit);
	}

	public double getNormalRotationY(AngleUnit unit) {
		return UnitConverter.convert(normalrotaionY, AngleUnit.RADIANT, unit);
	}

	public double getNormalRotationZ(AngleUnit unit) {
		return UnitConverter.convert(normalrotaionZ, AngleUnit.RADIANT, unit);
	}

	/**
	 * Getter for the xml base tag used for this (de-)serialization.
	 * 
	 * @return xml base tag
	 */
	public static String getBaseTag() {
		return "PLANE";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		// cleanup
		Elements elements = parent.getChildElements(TRANSLATION_TAG_NAME);
		for (int i = 0; i < elements.size(); i++) {
			parent.removeChild(elements.get(i));
		}
		elements = parent.getChildElements(ROTATION_TAG_NAME);
		for (int i = 0; i < elements.size(); i++) {
			parent.removeChild(elements.get(i));
		}

		Element transElement = new Element(TRANSLATION_TAG_NAME);
		parent.appendChild(transElement);
		transElement.addAttribute(new Attribute("x", String.valueOf(translationX)));
		transElement.addAttribute(new Attribute("y", String.valueOf(translationY)));
		transElement.addAttribute(new Attribute("z", String.valueOf(translationZ)));
		
		Element rotElement = new Element(ROTATION_TAG_NAME);
		parent.appendChild(rotElement);
		rotElement.addAttribute(new Attribute("x", String.valueOf(normalrotaionX)));
		rotElement.addAttribute(new Attribute("y", String.valueOf(normalrotaionY)));
		rotElement.addAttribute(new Attribute("z", String.valueOf(normalrotaionZ)));
	}

	/**
	 * Fills an {@link ObjectLocationData.Hypothesis} object from a given XOM {@link Element}.
	 * 
	 * @param hypothesisElement
	 *            The XOM {@link Element} to fill an object from.
	 * @param type
	 *            The {@link ObjectLocationData.Hypothesis} object to fill with all the information
	 *            given by the {@link Element} object.
	 * @throws ParsingException
	 */
	public static void fromElement(Element dataElement, PlaneData type) throws ParsingException {

		RegionData.fromElement(dataElement, type);

		Element transElement = dataElement.getFirstChildElement(TRANSLATION_TAG_NAME);
		double tx = ElementParser.getDoubleAttributeValue(transElement, "x");
		double ty = ElementParser.getDoubleAttributeValue(transElement, "y");
		double tz = ElementParser.getDoubleAttributeValue(transElement, "z");
		type.setTranslation(tx, ty, tz, LengthUnit.MILLIMETER);
		
		Element rotElement = dataElement.getFirstChildElement(ROTATION_TAG_NAME);
		double rx = ElementParser.getDoubleAttributeValue(rotElement, "x");
		double ry = ElementParser.getDoubleAttributeValue(rotElement, "y");
		double rz = ElementParser.getDoubleAttributeValue(rotElement, "z");
		type.setNormalRotationX(rx, ry, rz, AngleUnit.RADIANT);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return "[" + getBaseTag() + " timestamp: " + getTimestamp() + " scope: " + scope
				+ " Polygon: " + polygon + "]";

	}
}
