package de.unibi.airobots.btl.data;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Vector;

import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;
import nu.xom.ParsingException;

/**
 * 
 * @author lziegler
 */
public class PlaneList extends Type implements List<PlaneData> {

	/**
	 * 
	 */
	private static final long serialVersionUID = 5404915104164596108L;
	
	List<PlaneData> list = new Vector<PlaneData>();

	/**
	 * Default constructor.
	 */
	public PlaneList() {
	}

	/**
	 * Getter for the xml base tag used for this (de-)serialization.
	 * 
	 * @return xml base tag
	 */
	public static String getBaseTag() {
		return "PLANES";
	}
	
	/**
	 * Fills an {@link PlaneList} object from a given XOM
	 * {@link Element}.
	 * 
	 * @param objectElement
	 *            The XOM {@link Element} to fill an object from.
	 * @param type
	 *            The {@link PlaneList} object to fill with all the
	 *            information given by the {@link Element} object.
	 * @throws ParsingException 
	 */
	public static void fromElement(Element objectElement,
			PlaneList type) throws ParsingException {

		try {
			// check hypotheses
			Nodes nodes = objectElement.query(PlaneData.getBaseTag());
			for (int i = 0; i < nodes.size(); i++) {
				Node node = nodes.get(i);
				if (node instanceof Element) {

					PlaneData data = new PlaneData();
					PlaneData.fromElement((Element) node, data);
					type.add(data);
				}
			}

		} catch (NullPointerException ex) {
			// this happens when an element or attribute that is required is
			// not present
			throw new IllegalArgumentException("Missing element or attribute "
					+ "in document.", ex);
		}
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		for (PlaneData data : this) {
			Element dataElem = new Element(PlaneData.getBaseTag());
			data.fillInto(dataElem);
			parent.appendChild(dataElem);
		}
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		String text = "[PLANES\n";
		for (PlaneData data : list) {
			text += "\t" + data.toString() + "\n";
		}
		text += "]";
		return text;
	}

	@Override
	public boolean add(PlaneData e) {
		return list.add(e);
	}

	@Override
	public void add(int index, PlaneData element) {
		list.add(index, element);
	}

	@Override
	public boolean addAll(Collection<? extends PlaneData> c) {
		return list.addAll(c);
	}

	@Override
	public boolean addAll(int index, Collection<? extends PlaneData> c) {
		return list.addAll(index, c);
	}

	@Override
	public void clear() {
		list.clear();
	}

	@Override
	public boolean contains(Object o) {
		return list.contains(o);
	}

	@Override
	public boolean containsAll(Collection<?> c) {
		return list.containsAll(c);
	}

	@Override
	public PlaneData get(int index) {
		return list.get(index);
	}

	@Override
	public int indexOf(Object o) {
		return list.indexOf(o);
	}

	@Override
	public boolean isEmpty() {
		return list.isEmpty();
	}

	@Override
	public Iterator<PlaneData> iterator() {
		return list.iterator();
	}

	@Override
	public int lastIndexOf(Object o) {
		return list.lastIndexOf(o);
	}

	@Override
	public ListIterator<PlaneData> listIterator() {
		return list.listIterator();
	}

	@Override
	public ListIterator<PlaneData> listIterator(int index) {
		return list.listIterator(index);
	}

	@Override
	public boolean remove(Object o) {
		return list.remove(o);
	}

	@Override
	public PlaneData remove(int index) {
		return list.remove(index);
	}

	@Override
	public boolean removeAll(Collection<?> c) {
		return list.removeAll(c);
	}

	@Override
	public boolean retainAll(Collection<?> c) {
		return list.retainAll(c);
	}

	@Override
	public PlaneData set(int index, PlaneData element) {
		return list.set(index, element);
	}

	@Override
	public int size() {
		return list.size();
	}

	@Override
	public List<PlaneData> subList(int fromIndex, int toIndex) {
		return list.subList(fromIndex, toIndex);
	}

	@Override
	public Object[] toArray() {
		return list.toArray();
	}

	@Override
	public <T> T[] toArray(T[] a) {
		return list.toArray(a);
	}
}
