package de.unibi.airobots.btl.data;

import de.unibi.airobots.btl.common.Timestamp;
import de.unibi.airobots.btl.common.ElementParser;
import nu.xom.Attribute;
import nu.xom.Element;

/**
 * This class is used to store data from the sonar sensors from nao. It holds
 * the distances to objects in front of the left or right sensor.
 * 
 * @author sebschne
 */
public class SonarData extends Type {

	private double distanceRight;
	private double distanceLeft;

	public SonarData(double distanceRight, double distanceLeft) {
		super();
		this.distanceRight = distanceRight;
		this.distanceLeft = distanceLeft;
	}
	
	public static String getBaseTag() {
		return "SONARDATA";
	}
	
	/**
	 * 
	 * @return the observed distance to an object in front of the right sonar
	 *         sensor.
	 */
	public double getDistanceRight() {
		return distanceRight;
	}

	/**
	 * 
	 * @return the observed distance to an object in front of the left sonar
	 *         sensor.
	 */
	public double getDistanceLeft() {
		return distanceLeft;
	}

	public void setDistanceRight(double distanceRight) {
		this.distanceRight = distanceRight;
	}

	public void setDistanceLeft(double distanceLeft) {
		this.distanceLeft = distanceLeft;
	}
	
	public static void fromElement(Element element, SonarData type) {
		
		Type.fromElement(element, type);

		Element leftElement = element.getFirstChildElement("DISTLEFT");
		type.setDistanceLeft(ElementParser.getDoubleAttributeValue(leftElement, "value"));
		Element rightElement = element.getFirstChildElement("DISTRIGHT");
		type.setDistanceRight(ElementParser.getDoubleAttributeValue(rightElement, "value"));

	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {
		super.fillInto(parent);

		Element rightElement = new Element("DISTRIGHT");
		parent.appendChild(rightElement);
		rightElement.addAttribute(new Attribute("value", String.valueOf(getDistanceRight())));
		Element leftElement = new Element("DISTLEFT");
		parent.appendChild(leftElement);
		leftElement.addAttribute(new Attribute("value", String.valueOf(getDistanceLeft())));
		
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {

		return "Created: " + getTimestamp() + " Distance left:  "
				+ getDistanceLeft() + " Distance right " + getDistanceRight();

	}

}
