package de.unibi.airobots.btl.data;

import de.unibi.airobots.btl.common.ElementParser;
import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;

public class Voice extends Type implements Percept{

	@Override
	public String getPerceptName() {
		return "voice";
	}
	
	public static String getBaseTag() {
		return "VOICE";
	}
	
	public static void fromElement(Element element, Voice type) {
		
		Type.fromElement(element, type);
		
		Nodes nodes = element.query("*");
		for (int i = 0; i < nodes.size(); i++) {
			Node node = nodes.get(i);
			if (node instanceof Element) {
				Element elem = (Element) node;
				if (elem.getLocalName().equals("POSITION")) {
					type.setAngle((float)ElementParser
							.getDoubleAttributeValue(elem, "angle"));
					type.setAngleVariance((float)ElementParser
							.getDoubleAttributeValue(elem, "angleVariance"));
					type.setRuntimeDifference((float)ElementParser
							.getDoubleAttributeValue(elem, "runtimeDifference"));
					type.setRuntimeDifferenceVariance((float)ElementParser
							.getDoubleAttributeValue(elem, "runtimeDifferenceVariance"));
				}
			}
		}
		
	}
	
	public void fillInto(Element parent) {
		
		super.fillInto(parent);

		Element posElement = new Element("POSITION");
		parent.appendChild(posElement);
		posElement.addAttribute(new Attribute("" +
				"angle", String
				.valueOf(getAngle())));
		posElement.addAttribute(new Attribute("" +
				"angleVariance", String
				.valueOf(getAngleVariance())));
		posElement.addAttribute(new Attribute("" +
				"runtimeDifference", String
				.valueOf(getRuntimeDifference())));
		posElement.addAttribute(new Attribute("" +
				"runtimeDifferenceVariance", String
				.valueOf(getRuntimeDifferenceVariance())));
		
	}

	public void setAngle(float angle) {
		this.angle = angle;
	}

	public float getAngle() {
		return angle;
	}

	public void setRuntimeDifference(float runtimeDifference) {
		this.runtimeDifference = runtimeDifference;
	}

	public float getRuntimeDifference() {
		return runtimeDifference;
	}

	public void setAngleVariance(float angleVariance) {
		this.angleVariance = angleVariance;
	}

	public float getAngleVariance() {
		return angleVariance;
	}

	public void setRuntimeDifferenceVariance(float runtimeDifferenceVariance) {
		this.runtimeDifferenceVariance = runtimeDifferenceVariance;
	}

	public float getRuntimeDifferenceVariance() {
		return runtimeDifferenceVariance;
	}
	
	public boolean equals(Voice other){
		return (super.equals(other)) && (this.getAngle() == other.getAngle())
			&& (this.getAngleVariance() == other.getAngleVariance())
			&& (this.getRuntimeDifference() == other.getRuntimeDifference())
			&& (this.getRuntimeDifferenceVariance() == other.getRuntimeDifferenceVariance());
	}

	private float angle;
	private float runtimeDifference;
	private float angleVariance;
	private float runtimeDifferenceVariance;
	
}
