package de.unibi.airobots.btl.data.annotation;

import java.awt.Shape;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.util.LinkedList;

import org.apache.log4j.Logger;

import de.unibi.airobots.btl.data.NavigationGoalData;
import de.unibi.airobots.btl.data.PositionData;

/**
 * 
 * @author jhelbach
 */
public class MapRegion {

	public final static String POLYGONE_TYPE = "polygon";

	public final static String Rectangle_TYPE = "rectangle";

	private String type = "";

	private String label = "";

	private String group = "";

	private double height = 0;
	
	private LinkedList<MapPoint> points = new LinkedList<MapPoint>();

	private NavigationGoalData viewPoint;

	private Logger logger = Logger.getLogger(this.getClass());
	
	public MapRegion(String type) {
		this.type = type;
	}

	public MapRegion(String type, String label) {
		this.type = type;
		this.label = label;
	}

	public MapRegion(String type, String label, String group) {
		this.type = type;
		this.label = label;
		this.group = group;
	}

	public MapRegion(String type, String label, String group,
			LinkedList<MapPoint> points) {
		this.type = type;
		this.label = label;
		this.group = group;
		this.points = points;
	}

	public MapRegion(String type, String label, String group,
			LinkedList<MapPoint> points, NavigationGoalData viewPoint) {
		this.type = type;
		this.label = label;
		this.group = group;
		this.points = points;
		this.viewPoint = viewPoint;
	}
	
	public MapRegion(String type, String label, String group,
			LinkedList<MapPoint> points, NavigationGoalData viewPoint,
			double height) {
		this.type = type;
		this.label = label;
		this.group = group;
		this.points = points;
		this.viewPoint = viewPoint;
		this.height = height;
	}

	public void addPoint(int x, int y) {
		MapPoint p = new MapPoint(x, y);
		points.add(p);
	}

	public void addPoint(double x, double y) {
		MapPoint p = new MapPoint(x, y);
		points.add(p);
	}

	public void addPoint(MapPoint point) {
		points.add(point);
	}

	public Shape getShape() {
		GeneralPath generalPath = new GeneralPath();

		boolean firstPoint = true;
		logger.debug("points:");
		for (MapPoint p : points) {
			if (firstPoint) {
				generalPath.moveTo(p.getX(), p.getY());
				firstPoint = false;
			} else {
				generalPath.lineTo(p.getX(),p.getY());
				logger.debug("x " + (int) p.getX() + " y " + (int) p.getY() );
			}
		}

		return generalPath;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public String getLabel() {
		return label;
	}

	public void setLabel(String label) {
		this.label = label;
	}

	public String getGroup() {
		return group;
	}

	public void setGroup(String group) {
		this.group = group;
	}

	public LinkedList<MapPoint> getPoints() {
		return points;
	}

	public void setPoints(LinkedList<MapPoint> points) {
		this.points = points;
	}

	public NavigationGoalData getViewPoint() {
		return viewPoint;
	}

	public void setViewPoint(NavigationGoalData viewPoint) {
		this.viewPoint = viewPoint;
	}
	
	public double getHeight() {
		return height;
	}

	public void setHeight(double height) {
		this.height = height;
	}

	public void removePoint(double x, double y) {
		
		LinkedList<MapPoint> copy = new LinkedList<MapPoint>(points);
		
		for (MapPoint p: copy){
			
			if (p.getX() == x && p.getY() == y) {
				points.remove(p);
			}
		}
		
	}

	public int getPointCount() {
		return points.size();
	}

	
	/**
	 *  Returns {@link PositionData} for a Region. If the ViewPoint is set, the Viewpoint will be return otherwise 
	 *  the center of the bounding box. 
	 * 
	 * @return
	 */
	public PositionData getPositionData() {
		
		if (viewPoint != null) {
			return viewPoint;
		}

		Rectangle2D rect = getShape().getBounds2D();
		
		return new PositionData(rect.getCenterX(), rect.getCenterY(),0, System.currentTimeMillis());
	}

	public boolean contains(MapRegion region) {
		if (region == null) return false;
		
		Rectangle2D pr = region.getShape().getBounds2D();
		
		return getShape().contains(pr);
	}

	public boolean contains(double x, double y) {
		return getShape().contains(x, y);
		
	}

}
