package de.unibi.airobots.btl.data.asu;

/**
 * Contains all states of utterances.
 * @author malinke
 */
public enum ProcessingStatus {

	/**
	 * Understood lesser than 25% of the words.
	 */
	nonunderstanding,
	/**
	 * Understood up to 50% of the words.
	 */
	partialunderstanding,
	/**
	 * Understood more than 50% of the words.
	 */
	fullunderstanding;

	private static final float UNDERSTANDING_LIMIT = 0.25f;

	private static final float FULLUNDERSTANDING_LIMIT = 0.5f;

	/**
	 * Returns the fitting state of understanding / rating.
	 * @param rating of an linked hypothesis
	 * @return understanding state of utterance
	 */
	public static ProcessingStatus forRating(final float rating) {
		if (rating < UNDERSTANDING_LIMIT) {
			return nonunderstanding;
		}
		if (rating <= FULLUNDERSTANDING_LIMIT) {
			return partialunderstanding;
		}
		assert rating > FULLUNDERSTANDING_LIMIT : "state eval error";
		return fullunderstanding;

	}

}
