package de.unibi.airobots.btl.data.asu;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;

/**
 * Contains all categories of utterances.
 * @author malinke
 */
public enum UtteranceCategory {

	/**
	 * Instruction.
	 */
	instruction("task"),
	/**
	 * Description.
	 */
	description("description"),
	/**
	 * Socialisation.
	 */
	socialisation("greeting", "parting", "socialisation"),
	/**
	 * Query.
	 */
	query("question"),
	/**
	 * Object.
	 */
	object("object"),
	/**
	 * Grip.
	 */
	grip("grip"),
	/**
	 * Deletion.
	 */
	deletion("deletion"),
	/**
	 * Confirmation.
	 */
	confirmation("confirmation"),
	/**
	 * Negation.
	 */
	negation("negation"),
	/**
	 * Serves as fallback category. Will be returned, if no other is found.
	 */
	fragment();

	private static Map<String, UtteranceCategory> catMap;

	private final String[] frameNames;

	private UtteranceCategory(final String... frameNames) {
		this.frameNames = frameNames;
	}

	/**
	 * Returns the category of a frame.
	 * 
	 * @param frameName
	 *            the name of the frame
	 * @return fitting category
	 */
	public static UtteranceCategory categoryOf(final String frameName) {
		if (catMap == null) {
			buildMap();
		}
		final String lowerName = frameName.toLowerCase();
		if (catMap.containsKey(lowerName)) {
			return catMap.get(lowerName);
		}
		Logger.getLogger(UtteranceCategory.class.getName()).severe(
				"No category for " + frameName + ". --> fragment.");
		return fragment;
	}

	private static void buildMap() {
		catMap = new HashMap<String, UtteranceCategory>();
		for (UtteranceCategory c : values()) {
			for (String frameName : c.frameNames) {
				catMap.put(frameName, c);
			}
		}
	}

}
