package de.unibi.airobots.btl.data.speechrec;


/**
 * Domain class representing the grammar tree provided by the speech
 * recognizer. The GrammarTree itself is kind of a pseudo start symbol,
 * since the real start symbol might be hidden in the speech recognizers
 * output. Furthermore the GrammarTree can also include partially parsed
 * utterances e.g. if the parser had to cancel and reinitialize to parse the
 * utterance.
 * 
 * @author lschilli
 * 
 */
public class GrammarTree extends GrammarNonTerminal {

	private boolean cancel;
	private boolean fault;
	private boolean skip;

	public GrammarTree() {
		super(null);
	}

	/**
	 * Indicates the parser has at least once canceled (and reinitialized)
	 * during parsing
	 * 
	 * @return
	 */
	public boolean isCancel() {
		return cancel;
	}

	public void setCancel(boolean cancel) {
		this.cancel = cancel;
	}

	/**
	 * Indicates the parser has rated at least one word as a fault (not
	 * matching to the grammar structure).
	 * 
	 * @return
	 */
	public boolean isFault() {
		return fault;
	}

	public void setFault(boolean fault) {
		this.fault = fault;
	}

	/**
	 * Indicates the parser has skipped at least one word (if allowed to).
	 * 
	 * @return
	 */
	public boolean isSkip() {
		return skip;
	}

	public void setSkip(boolean skip) {
		this.skip = skip;
	}

}