package de.unibi.airobots.btl.data;

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.common.Timestamp;

/**
 * Test cases for {@link Type}.
 * 
 * @author lziegler
 */
public class CameraPositionDataTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws Exception {

		CameraPositionData original = new CameraPositionData();
		final long timestamp = System.currentTimeMillis();
		original.setTimestamp(timestamp);
		original.setGenerator("test");
		original.setPan(0.346);
		original.setTilt(123123.435);
		original.setZoom(0.33);

		Document doc = new Document(new Element(CameraPositionData.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		CameraPositionData parsed = new CameraPositionData();
		CameraPositionData.fromElement(doc.getRootElement(), parsed);

		assertEquals(original.getGenerator(), parsed.getGenerator());
		assertEquals(original.getTimestamp(), parsed.getTimestamp());
		assertEquals(original.getPan(), parsed.getPan(), 0.00001);
		assertEquals(original.getTilt(), parsed.getTilt(), 0.00001);
		assertEquals(original.getZoom(), parsed.getZoom(), 0.00001);

	}

	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("CameraPosition.xml"));

		CameraPositionData parsed = new CameraPositionData();
		CameraPositionData.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(), CameraPositionData
				.getBaseTag());

		assertEquals("test", parsed.getGenerator());
		assertEquals(new Timestamp(12345, 67890), parsed.getTimestamp());
		assertEquals(0.345, parsed.getPan(), 0.00001);
		assertEquals(123, parsed.getTilt(), 0.00001);
		assertEquals(0.75, parsed.getZoom(), 0.00001);

	}

}
