package de.unibi.airobots.btl.data;

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.common.Timestamp;

/**
 * Test cases for {@link Type}.
 * 
 * @author jwienke
 */
public class CameraTargetDataTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws Exception {

		CameraTargetData original = new CameraTargetData();
		final long timestamp = System.currentTimeMillis();
		original.setTimestamp(timestamp);
		original.setGenerator("test");
		original.setAzimuth(0.346);
		original.setDistance(123123.435);
		original.setHeight(231434.56);
		original.setZoom(0.33);

		Document doc = new Document(new Element(CameraTargetData.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		CameraTargetData parsed = new CameraTargetData();
		CameraTargetData.fromElement(doc.getRootElement(), parsed);

		assertEquals(original.getGenerator(), parsed.getGenerator());
		assertEquals(original.getTimestamp(), parsed.getTimestamp());
		assertEquals(original.getAzimuth(), parsed.getAzimuth(), 0.00001);
		assertEquals(original.getDistance(), parsed.getDistance(), 0.00001);
		assertEquals(original.getHeight(), parsed.getHeight(), 0.00001);
		assertEquals(original.getZoom(), parsed.getZoom(), 0.00001);

	}

	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("CameraTarget.xml"));

		CameraTargetData parsed = new CameraTargetData();
		CameraTargetData.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(), CameraTargetData
				.getBaseTag());

		assertEquals("test", parsed.getGenerator());
		assertEquals(new Timestamp(12345, 67890), parsed.getTimestamp());
		assertEquals(0.345, parsed.getAzimuth(), 0.00001);
		assertEquals(123, parsed.getDistance(), 0.00001);
		assertEquals(2342, parsed.getHeight(), 0.00001);
		assertEquals(0.75, parsed.getZoom(), 0.00001);

	}

}
