package de.unibi.airobots.btl.data;

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.common.Timestamp;
import de.unibi.airobots.btl.data.GlobalPlannerStateChange.PlannerState;

/**
 * Test cases for {@link Type}.
 * 
 * @author jwienke
 */
public class GlobalPlannerStateChangeTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws Exception {

		GlobalPlannerStateChange original = new GlobalPlannerStateChange();
		final long timestamp = System.currentTimeMillis();
		original.setTimestamp(timestamp);
		original.setGenerator("test");
		original.setNewState(PlannerState.PATH_BLOCKED);

		Document doc = new Document(new Element(GlobalPlannerStateChange
				.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		GlobalPlannerStateChange parsed = new GlobalPlannerStateChange();
		GlobalPlannerStateChange.fromElement(doc.getRootElement(), parsed);

		assertEquals(original.getGenerator(), parsed.getGenerator());
		assertEquals(original.getTimestamp(), parsed.getTimestamp());
		assertEquals(original.getNewState(), parsed.getNewState());

	}

	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("GlobalPlannerStateChange.xml"));

		GlobalPlannerStateChange parsed = new GlobalPlannerStateChange();
		GlobalPlannerStateChange.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(),
				GlobalPlannerStateChange.getBaseTag());

		assertEquals("test", parsed.getGenerator());
		assertEquals(new Timestamp(12345, 67890), parsed.getTimestamp());
		assertEquals(PlannerState.PLANNING, parsed.getNewState());
	}

}
