package de.unibi.airobots.btl.data;

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.common.MicroTimestamp;
import de.unibi.airobots.btl.common.Timestamp;

/**
 * Test cases for {@link Type}.
 * 
 * @author jwienke
 */
public class LaserDataTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws Exception {

		LaserData original = new LaserData();
		final long timestamp = System.currentTimeMillis();
		original.setTimestamp(timestamp);
		original.setGenerator("test");
		original.setReadTime(new MicroTimestamp(123, 456));
		double scans[] = new double[] { 0.5, 12.4, 10.4, -5.3, 0.01 };
		original.setScanValues(scans);

		Document doc = new Document(new Element(LaserData.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		LaserData parsed = new LaserData();
		LaserData.fromElement(doc.getRootElement(), parsed);

		assertEquals(original.getGenerator(), parsed.getGenerator());
		assertEquals(original.getTimestamp(), parsed.getTimestamp());
		assertEquals(original.getReadTime(), parsed.getReadTime());
		assertEquals(original.getNumLaserPoints(), parsed.getNumLaserPoints());
		for (int i = 0; i < original.getNumLaserPoints(); i++) {
			assertEquals(original.getScanValues()[i],
					parsed.getScanValues()[i], 0.0001);
		}

	}

	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils.makeTestFileName("Laser.xml"));

		LaserData parsed = new LaserData();
		LaserData.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(), LaserData
				.getBaseTag());

		assertEquals("test", parsed.getGenerator());
		assertEquals(new Timestamp(12345, 67890), parsed.getTimestamp());
		assertEquals(5, parsed.getNumLaserPoints());
		double originalValues[] = new double[] { 0.01, 0.002, 0.003, 0.004, 0.005 };
		for (int i = 0; i < parsed.getNumLaserPoints(); i++) {
			assertEquals(originalValues[i],
					parsed.getScanValues()[i], 0.0001);
		}

	}

}
