package de.unibi.airobots.btl.data;

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.common.Polygon;
import de.unibi.airobots.btl.common.Timestamp;

/**
 * Test cases for {@link Type}.
 * 
 * @author lziegler
 */
public class ObjectLocationListTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws Exception {

		final long timestamp = System.currentTimeMillis();

		Polygon poly = new Polygon();
		poly.addPoint(0, 0);
		poly.addPoint(1, 1);
		poly.addPoint(2, 2);

		ObjectLocationList originalList = new ObjectLocationList();
		ObjectLocationData original = new ObjectLocationData();
		original.setClassLabel("testclass");

		ObjectLocationData.Hypothesis originalHyp = new ObjectLocationData.Hypothesis();

		originalHyp.setTimestamp(timestamp);
		originalHyp.setGenerator("test");
		originalHyp.setReliability(0.1);
		originalHyp.setPolygon(poly);
		originalHyp.setSensorType("SONY_EVI");

		original.addHypothesis(originalHyp);
		originalList.add(original);

		Document doc = new Document(
				new Element(ObjectLocationList.getBaseTag()));
		originalList.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		ObjectLocationList parsedList = new ObjectLocationList();
		ObjectLocationList.fromElement(doc.getRootElement(), parsedList);
		ObjectLocationData parsed = parsedList.get(0);

		ObjectLocationData.Hypothesis parsedHyp = parsed
				.getLocationHypotheses().iterator().next();

		assertEquals(originalHyp.getGenerator(), parsedHyp.getGenerator());
		assertEquals(originalHyp.getTimestamp(), parsedHyp.getTimestamp());
		assertEquals(original.getClassLabel(), parsed.getClassLabel());
		assertEquals(originalHyp.getReliability(), parsedHyp.getReliability(),
				0.0001);
		assertEquals(originalHyp.getSensorType(), parsedHyp.getSensorType());
		assertEquals(originalHyp.getPolygon(), parsedHyp.getPolygon());

		System.out.println(parsedList);

	}

	@Test
	public void fileCompatibility() throws Exception {

		Polygon poly = new Polygon();
		poly.addPoint(0, 0);
		poly.addPoint(1, 1);
		poly.addPoint(2, 2);

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("ObjectLocationList.xml"));

		ObjectLocationList parsedList = new ObjectLocationList();
		ObjectLocationList.fromElement(doc.getRootElement(), parsedList);
		ObjectLocationData parsed = parsedList.get(0);
		ObjectLocationData.Hypothesis parsedHyp = parsed
				.getLocationHypotheses().iterator().next();

		assertEquals(ObjectLocationList.getBaseTag(), doc.getRootElement()
				.getLocalName());

		assertEquals("BoostDetector", parsedHyp.getGenerator());
		assertEquals(new Timestamp(253543797l, 253543712l), parsedHyp
				.getTimestamp());
		assertEquals("smacks", parsed.getClassLabel());
		assertEquals(0.6, parsedHyp.getReliability(), 0.002);
		assertEquals("SONY_EVI", parsedHyp.getSensorType());
		assertEquals(poly, parsedHyp.getPolygon());

	}

}
