package de.unibi.airobots.btl.data;

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.common.Timestamp;

/**
 * Test cases for {@link Type}.
 * 
 * @author jwienke
 */
public class PersonDataTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() {

		PersonData original = new PersonData();
		final long timestamp = System.currentTimeMillis();
		original.setTimestamp(timestamp);
		original.setGenerator("test");
		original.setAngle(-0.345);
		original.setDistance(21.344);
		original.setFacing(true);
		original.setGazing(4.993);
		original.setId(1234);
		original.setName("peter");
		original.setNeighbourDistance(3.45);
		original.setNeighbourId(42);
		original.setPoi(false);
		original.setTalking(true);
		original.setWalking(false);

		Document doc = new Document(new Element(PersonData.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		PersonData parsed = new PersonData();
		PersonData.fromElement(doc.getRootElement(), parsed);

		assertEquals(original.getGenerator(), parsed.getGenerator());
		assertEquals(original.getTimestamp(), parsed.getTimestamp());
		assertEquals(original.getAngle(), parsed.getAngle(), 0.0001);
		assertEquals(original.getDistance(), parsed.getDistance(), 0.00001);
		assertEquals(original.isFacing(), parsed.isFacing());
		assertEquals(original.getGazing(), parsed.getGazing(), 0.00001);
		assertEquals(original.getId(), parsed.getId());
		assertEquals(original.getName(), parsed.getName());
		assertEquals(original.getNeighbourDistance(), parsed
				.getNeighbourDistance(), 0.0001);
		assertEquals(original.getNeighbourId(), parsed.getNeighbourId());
		assertEquals(original.isPoi(), parsed.isPoi());
		assertEquals(original.isTalking(), parsed.isTalking());
		assertEquals(original.isWalking(), parsed.isWalking());

	}
	
	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("Person.xml"));

		PersonData parsed = new PersonData();
		PersonData.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(), PersonData
				.getBaseTag());

		assertEquals("test", parsed.getGenerator());
		assertEquals(new Timestamp(12345, 67890), parsed.getTimestamp());
		assertEquals(2.1467, parsed.getAngle(), 0.0001);
		assertEquals(1235, parsed.getDistance(), 0.00001);
		assertEquals(false, parsed.isFacing());
		assertEquals(Math.PI / 2.0, parsed.getGazing(), 0.00001);
		assertEquals(123, parsed.getId());
		assertEquals("peter", parsed.getName());
		assertEquals(456, parsed
				.getNeighbourDistance(), 0.0001);
		assertEquals(12, parsed.getNeighbourId());
		assertEquals(true, parsed.isPoi());
		assertEquals(true, parsed.isTalking());
		assertEquals(true, parsed.isWalking());

	}

}
