package de.unibi.airobots.btl.data;

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.common.Timestamp;

/**
 * Test cases for {@link Type}.
 * 
 * @author ttoenige
 */
public class PoseDataTest {
	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws Exception {
		PoseData original = new PoseData();
		final long timestamp = System.currentTimeMillis();
		original.setTimestamp(timestamp);
		original.setGenerator("test");
		original.setX(111);
		original.setY(222);
		original.setZ(333);

		Document doc = new Document(new Element(PoseData.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		PoseData parsed = new PoseData();
		PoseData.fromElement(doc.getRootElement(), parsed);

		assertEquals(original.getGenerator(), parsed.getGenerator());
		assertEquals(original.getTimestamp(), parsed.getTimestamp());
		assertEquals(original.getX(), parsed.getX(), 0.00001);
		assertEquals(original.getY(), parsed.getY(), 0.00001);
		assertEquals(original.getZ(), parsed.getZ(), 0.00001);
	}

	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils.makeTestFileName("PoseData.xml"));

		PoseData parsed = new PoseData();
		PoseData.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(), PoseData.getBaseTag());

		assertEquals("test", parsed.getGenerator());
		assertEquals(new Timestamp(12345, 67890), parsed.getTimestamp());
		assertEquals(18, parsed.getX(), 0.00001);
		assertEquals(25, parsed.getY(), 0.00001);
		assertEquals(34, parsed.getZ(), 0.00001);
	}

}
