package de.unibi.airobots.btl.data;

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.common.Point3D;
import de.unibi.airobots.btl.data.SRCalibratedData.Scope;
import de.unibi.airobots.btl.units.LengthUnit;

/**
 * Test cases for {@link Type}.
 * 
 * @author lziegler
 */
public class SRCalibratedDataTest {

	@Test
	public void selfCompatibility() throws Exception {

		SRCalibratedData data0 = new SRCalibratedData(Scope.GLOBAL);
		Point3D p0 = new Point3D(1, 2, 3, LengthUnit.MILLIMETER, Point3D.Scope.GLOBAL);
		Point3D p1 = new Point3D(1, 2, 3, LengthUnit.MILLIMETER, Point3D.Scope.GLOBAL);
		Point3D p2 = new Point3D(0.1, 0.2, 0.3, LengthUnit.MILLIMETER,
				Point3D.Scope.LOCAL);
		data0.addPoint(p0);
		data0.addPoint(p1);
		data0.addPoint(p2);

		Element element = new Element(SRCalibratedData.getBaseTag());
		data0.fillInto(element);

		SRCalibratedData data1 = new SRCalibratedData();
		SRCalibratedData.fromElement(element, data1);

		assertEquals(data0.size(), data1.size());
		assertEquals(data0.getScope(), data1.getScope());
		assertEquals(data0.getData().get(0), data1.getData().get(0));
		assertEquals(data0.getData().get(1), data1.getData().get(1));
	}

	@Test
	public void fileCompatibility() throws Exception {

		SRCalibratedData data0 = new SRCalibratedData(Scope.GLOBAL);
		Point3D p0 = new Point3D(1, 2, 3, LengthUnit.MILLIMETER, Point3D.Scope.GLOBAL);
		Point3D p1 = new Point3D(1, 2, 3, LengthUnit.MILLIMETER, Point3D.Scope.GLOBAL);
		Point3D p2 = new Point3D(0.1f, 0.2f, 0.3f, LengthUnit.MILLIMETER,
				Point3D.Scope.LOCAL);
		data0.addPoint(p0);
		data0.addPoint(p1);
		data0.addPoint(p2);

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("SRCalibratedData.xml"));

		SRCalibratedData data1 = new SRCalibratedData();
		SRCalibratedData.fromElement(doc.getRootElement(), data1);

		assertEquals(data0.size(), data1.size());
		assertEquals(data0.getScope(), data1.getScope());
		assertEquals(data0.getData().get(0), data1.getData().get(0));
		assertEquals(data0.getData().get(1), data1.getData().get(1));
	}
}
