package de.unibi.airobots.btl.data;

import static org.junit.Assert.*;

import java.util.LinkedList;
import java.util.List;

import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.common.Timestamp;
import de.unibi.airobots.btl.data.SlamUpdateData.SlamUpdateGrid;

/**
 * Test cases for {@link Type}.
 * 
 * @author jwienke
 */
public class SlamUpdateDataTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() throws Exception {

		SlamUpdateData original = new SlamUpdateData();
		final long timestamp = System.currentTimeMillis();
		original.setTimestamp(timestamp);
		original.setGenerator("test");
		List<SlamUpdateData.SlamUpdateGrid> additions = new LinkedList<SlamUpdateGrid>();
		additions.add(original.new SlamUpdateGrid(12.4, -3.07));
		additions.add(original.new SlamUpdateGrid(2.34, 53.07));
		original.setAdditions(additions);
		List<SlamUpdateData.SlamUpdateGrid> removals = new LinkedList<SlamUpdateGrid>();
		removals.add(original.new SlamUpdateGrid(142.4, -3.207));
		removals.add(original.new SlamUpdateGrid(2.234, -43.07));
		removals.add(original.new SlamUpdateGrid(29.234, -7.07));
		original.setRemovals(removals);
		original.setType("init");

		Document doc = new Document(new Element(SlamUpdateData.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		SlamUpdateData parsed = new SlamUpdateData();
		SlamUpdateData.fromElement(doc.getRootElement(), parsed);

		assertEquals(original.getGenerator(), parsed.getGenerator());
		assertEquals(original.getTimestamp(), parsed.getTimestamp());
		assertEquals(original.getAdditions(), parsed.getAdditions());
		assertEquals(original.getRemovals(), parsed.getRemovals());
		assertEquals(original.getType(), parsed.getType());

	}

	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("SlamUpdate.xml"));

		SlamUpdateData parsed = new SlamUpdateData();
		SlamUpdateData.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(), SlamUpdateData
				.getBaseTag());

		assertEquals("test", parsed.getGenerator());
		assertEquals(new Timestamp(12345, 67890), parsed.getTimestamp());

		assertEquals(2, parsed.getAdditions().size());
		assertEquals(3, parsed.getRemovals().size());

	}

	@Test
	public void genericFromDocument() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("SlamUpdate.xml"));

		SlamUpdateData parsed = SlamUpdateData.fromDocument(doc,
				SlamUpdateData.class);

		assertEquals("test", parsed.getGenerator());
		assertEquals(new Timestamp(12345, 67890), parsed.getTimestamp());

		assertEquals(2, parsed.getAdditions().size());
		assertEquals(3, parsed.getRemovals().size());
		
	}

}
