package de.unibi.airobots.btl.data.asu;

import java.util.List;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 * Tests FrameEntry.
 * 
 * @author malinke
 */
public final class FrameEntryTest {

	/**
	 * Test of getName method, of class FrameEntry.
	 */
	@Test
	public void testGetName() {
		String name = "ASU";
		FrameEntry frame = new FrameEntry(name);
		assertEquals(name, frame.getName());
	}

	/**
	 * Test of getTopName method, of class FrameEntry.
	 */
	@Test
	public void testGetTopName() {
		String name = "ASU";
		FrameEntry frame = new FrameEntry(name);
		frame.setTopFrame(frame);
		assertEquals(name, frame.getTopName());
	}

	/**
	 * Test of slotList method, of class FrameEntry.
	 */
	@Test
	public void testAllRelatedFrames() {
		FrameEntry root = new FrameEntry("0");
		FrameEntry child1 = new FrameEntry("1");
		FrameEntry child2 = new FrameEntry("2");
		root.addSlot(child1);
		root.addSlot(child2);
		List<FrameEntry> allRelated = root.slotList();
		assertEquals(2, allRelated.size());
		assertEquals(child1, allRelated.get(0));
		assertEquals(child2, allRelated.get(1));
	}

	/**
	 * Tests comparison with more general frames.
	 */
	@Test
	public void testIsTypeOf() {
		FrameEntry polygon = new FrameEntry("polygon");
		FrameEntry rectangle = new FrameEntry("rectangle");
		FrameEntry square = new FrameEntry("square");
		rectangle.setTopFrame(polygon);
		square.setTopFrame(rectangle);
		assertTrue(square.isOfType(square));
		assertFalse(rectangle.isOfType(square));
		assertTrue(square.isOfType(rectangle));
		assertTrue(square.isOfType(polygon));
	}

}